@@ -5,7 +5,9 @@
 QSDsan: Quantitative Sustainable Design for sanitation and resource recovery systems
 
 This module is developed by:
-    Yalin Li <zoe.yalin.li@gmail.com>
+
+    Yalin Li <mailto.yalin.li@gmail.com>
+
     Joy Zhang <joycheung1994@gmail.com>
 
 This module is under the University of Illinois/NCSA Open Source License.
@@ -18,7 +20,7 @@ import sys
 py_version = sys.version.split('.')
 _PY_MAJOR, _PY_MINOR = int(py_version[0]), int(py_version[1])
 
-if (_PY_MAJOR, _PY_MINOR) <= (3, 7):
+if (_PY_MAJOR, _PY_MINOR) <= (3, 7): # pragma: no cover
     from warnings import warn
     if (_PY_MAJOR, _PY_MINOR) >= (3, 5):
         try: import pickle5 as _pk
@@ -46,27 +48,38 @@ except pkg_resources.DistributionNotFound:  # pragma: no cover
 del sys, py_version, pkg_resources
 
 
+# BioSTEAM/Thermosteam APIs
 import thermosteam as tmo
 import biosteam as bst
 Chemical = tmo.Chemical
 Chemicals = tmo.Chemicals
 CompiledChemicals = tmo.CompiledChemicals
 Stream = tmo.Stream
+MultiStream = tmo.MultiStream
 set_thermo = tmo.settings.set_thermo
 get_components = tmo.settings.get_chemicals
 get_thermo = tmo.settings.get_thermo
 
+HeatUtility = bst.HeatUtility
 PowerUtility = bst.PowerUtility
-MultiStream = tmo.MultiStream
 Unit = bst.Unit
 System = bst.System
+Scope = bst.utils.Scope
 Model = bst.Model
+Metric = bst.Metric
+Parameter = bst.Parameter
 Flowsheet = bst.Flowsheet
+main_flowsheet = bst.main_flowsheet
 CEPCI = bst.CE # Chemical Engineering Plant Cost Index
 CEPCI_by_year = bst.units.design_tools.CEPCI_by_year
 del tmo, bst
 
+
+# Global variables
 currency = 'USD'
+CHECK_IMPACT_INDICATOR_CONSISTENCY = True
+CHECK_IMPACT_ITEM_CONSISTENCY = True
+
 
 from . import utils
 from ._component import *
@@ -80,26 +93,26 @@ from ._construction import *
 from ._equipment import *
 from ._transportation import *
 from ._sanunit import *
-from ._simple_tea import *
+from ._tea import *
 from ._lca import *
 
 
 from . import (
     _component,
     _components,
-    _sanstream,
-    _waste_stream,
-    _process,
-    _impact_indicator,
-    _impact_item,
     _construction,
     _equipment,
-    _transportation,
-    _sanunit,
-    _simple_tea,
+    _impact_indicator,
+    _impact_item,
     _lca,
-    processes,
+    _process,
+    _sanstream,
+    _sanunit,
+    _tea,
+    _transportation,
+    _waste_stream,
     equipments,
+    processes,
     sanunits,
     stats,
     )
@@ -108,18 +121,24 @@ utils._secondary_importing()
 for _slot in utils.doc_examples.__all__:
     setattr(utils, _slot, getattr(utils.doc_examples, _slot))
 
+# Add the `pump` decorator to the util module
+from .sanunits import wwtpump
+utils.__all__ = (*utils.__all__, 'wwtpump')
+setattr(utils, 'wwtpump', wwtpump)
+
+
 __all__ = (
     *_component.__all__,
     *_components.__all__,
-    *_sanstream.__all__,
-    *_waste_stream.__all__,
-    *_process.__all__,
+    *_construction.__all__,
+    *_equipment.__all__,
     *_impact_indicator.__all__,
     *_impact_item.__all__,
-    *_construction.__all__,
-    *_transportation.__all__,
-    *_equipment.__all__,
-    *_sanunit.__all__,
-    *_simple_tea.__all__,
     *_lca.__all__,
-           )
\ No newline at end of file
+    *_process.__all__,
+    *_sanstream.__all__,
+    *_sanunit.__all__,
+    *_tea.__all__,
+    *_transportation.__all__,
+    *_waste_stream.__all__,
+    )
\ No newline at end of file
