@@ -5,8 +5,8 @@
 QSDsan: Quantitative Sustainable Design for sanitation and resource recovery systems
 
 This module is developed by:
-    Joy Cheung <joycheung1994@gmail.com>
-    Yalin Li <zoe.yalin.li@gmail.com>
+    Joy Zhang <joycheung1994@gmail.com>
+    Yalin Li <mailto.yalin.li@gmail.com>
 
 Part of this module is based on the Thermosteam package:
 https://github.com/BioSTEAMDevelopmentGroup/thermosteam
@@ -18,6 +18,7 @@ for license details.
 
 
 import thermosteam as tmo
+from thermo import TDependentProperty
 from chemicals.elements import (
     mass_fractions as get_mass_frac,
     molecular_weight,
@@ -25,6 +26,7 @@ from chemicals.elements import (
     )
 from . import Chemical
 from .utils import auom, copy_attr, cod_test_stoichiometry, electron_acceptor_cod
+from warnings import warn
 
 __all__ = ('Component',)
 
@@ -63,7 +65,8 @@ def component_identity(component, pretty=False):
 # Will stored as an array when compiled
 _num_component_properties = ('i_C', 'i_N', 'i_P', 'i_K', 'i_Mg', 'i_Ca',
                              'i_mass', 'i_charge', 'i_COD', 'i_NOD',
-                             'f_BOD5_COD', 'f_uBOD_COD', 'f_Vmass_Totmass', )
+                             'f_BOD5_COD', 'f_uBOD_COD', 'f_Vmass_Totmass',
+                             'chem_MW')
 
 # Fields that cannot be left as None
 _key_component_properties = ('particle_size', 'degradability', 'organic',
@@ -94,8 +97,6 @@ component_units_of_measure = {
 
 # %%
 
-#!!! What should the gas/solid-phase Component value (e.g., CH4 in biogas)
-# for particle_size and degradability? Can we put None or NA?
 allowed_values = {
     'particle_size': ('Dissolved gas', 'Soluble', 'Colloidal', 'Particulate'),
     'degradability': ('Readily', 'Slowly', 'Undegradable'),
@@ -111,8 +112,8 @@ def check_return_property(name, value):
         if name.startswith('f_') and (value>1 or value<0):
             raise ValueError(f'{name} must be within [0,1].')
     elif name in allowed_values.keys():
-        assert value in allowed_values[name], \
-            f'{name} must be in {allowed_values[name]}.'
+        if value not in allowed_values[name]:
+            raise ValueError(f'{name} must be in {allowed_values[name]}.')
         return value
 
 # =============================================================================
@@ -124,6 +125,8 @@ class Component(Chemical):
     A subclass of :class:`thermosteam.Chemical` with additional attributes
     and methods for waste treatment.
 
+    At a minimum, ID, particle_size, degradability, and organic should be provided.
+
     Parameters
     ----------
     ID : str
@@ -195,7 +198,7 @@ class Component(Chemical):
 
     Examples
     --------
-    `Component <https://qsdsan.readthedocs.io/en/latest/tutorials/Component.html>`_
+    `Component <https://qsdsan.readthedocs.io/en/latest/tutorials/2_Component.html>`_
 
 
     See Also
@@ -215,13 +218,19 @@ class Component(Chemical):
         if search_ID:
             self = super().__new__(cls, ID=ID, search_ID=search_ID,
                                    search_db=True, **chemical_properties)
-        else:
-            self = super().__new__(cls, ID=ID, search_db=False, **chemical_properties)
+        else: # still try to search nonetheless
+            try: self = super().__new__(cls, ID=ID, **chemical_properties)
+            except LookupError:
+                self = super().__new__(cls, ID=ID, search_db=False, **chemical_properties)
 
         self._ID = ID
+        self._chem_MW = 1
         if formula:
             self._formula = None
             self.formula = formula
+        else:
+            if self.formula:
+                self._chem_MW = molecular_weight(self.atoms)
         if phase: lock_phase(self, phase)
 
         self._measured_as = measured_as
@@ -327,11 +336,11 @@ class Component(Chemical):
                 if self.measured_as in self.atoms:
                     i = 1/get_mass_frac(self.atoms)[self.measured_as]
                 elif self.measured_as == 'COD':
-                    chem_MW = molecular_weight(self.atoms)
+                    # chem_MW = molecular_weight(self.atoms)
                     chem_charge = charge_from_formula(self.formula)
                     Cr2O7 = - cod_test_stoichiometry(self.atoms, chem_charge)['Cr2O7-2']
                     cod = Cr2O7 * 1.5 * molecular_weight({'O':2})
-                    i = chem_MW/cod
+                    i = self.chem_MW/cod
                 elif self.measured_as:
                     raise AttributeError(f'Must specify i_mass for component {self.ID} '
                                          f'measured as {self.measured_as}.')
@@ -342,7 +351,7 @@ class Component(Chemical):
             i = 1
         self._i_mass = check_return_property('i_mass', i)
 
-    #!!! Need to enable calculation from formula and water chemistry equilibria
+    #TODO: Need to enable calculation from formula and water chemistry equilibria
     @property
     def i_charge(self):
         '''
@@ -356,8 +365,7 @@ class Component(Chemical):
         if not self._i_charge:
             if self.formula:
                 charge = charge_from_formula(self.formula)
-                chem_MW = molecular_weight(self.atoms)
-                i = charge/chem_MW * self.i_mass
+                i = charge/self.chem_MW * self.i_mass
                 self._i_charge = check_return_property('i_charge', i)
             else: self._i_charge = 0.
 
@@ -428,14 +436,43 @@ class Component(Chemical):
                 raise AttributeError(f"Component {self.ID} must be measured as "
                                      f"either COD or one of its constituent atoms, "
                                      f"if not as itself.")
+        else:
+            # if self.atoms: self._MW = molecular_weight(self.atoms)
+            # else: self._MW = 1
+            self._MW = self.chem_MW
 
         if self._measured_as != measured_as:
             self._convert_i_attr(measured_as)
 
         self._measured_as = measured_as
 
+    formula = property(Chemical.formula.fget)
+    @formula.setter
+    def formula(self, formula):
+        Chemical.formula.fset(self, formula)
+        self._chem_MW = molecular_weight(self.atoms)
+        if hasattr(self, '_measured_as'):
+            _ms = self._measured_as
+            self.measured_as = None
+            for field in _num_component_properties:
+                if field.startswith('i_'):
+                    setattr(self, field, None)
+            self.measured_as = _ms
+
+    @property
+    def chem_MW(self):
+        return self._chem_MW
+    @chem_MW.setter
+    def chem_MW(self, MW):
+        if self.formula:
+            warn(f'Component {self.ID} has a formula {self.formula}, '
+                 f'ignored chem_MW setting')
+        else:
+            if MW >= 0: self._chem_MW = MW
+            else: raise ValueError('chem_MW cannot be less than 0')
+
     def _convert_i_attr(self, new):
-        if new == None:
+        if new is None:
             denom = self._i_mass
         elif new == 'COD':
             denom = self._i_COD
@@ -493,7 +530,7 @@ class Component(Chemical):
         return self._i_COD or 0.
     @i_COD.setter
     def i_COD(self, i):
-        if i != None: self._i_COD = check_return_property('i_COD', i)
+        if i is not None: self._i_COD = check_return_property('i_COD', i)
         else:
             if self.organic or self.formula in ('H2', 'O2', 'N2', 'NO2-', 'NO3-'):
                 if self.measured_as == 'COD': self._i_COD = 1.
@@ -501,14 +538,14 @@ class Component(Chemical):
                     raise AttributeError(f"Must specify `i_COD` for organic component {self.ID}, "
                                          f"which is not measured as COD and has no formula.")
                 else:
-                    chem_MW = molecular_weight(self.atoms)
+                    # chem_MW = molecular_weight(self.atoms)
                     chem_charge = charge_from_formula(self.formula)
                     if self.formula in ('O2', 'N2', 'NO2-', 'NO3-'):
                         cod = electron_acceptor_cod(self.atoms, chem_charge) * molecular_weight({'O':2})
                     else:
                         Cr2O7 = - cod_test_stoichiometry(self.atoms, chem_charge)['Cr2O7-2']
                         cod = Cr2O7 * 1.5 * molecular_weight({'O':2})
-                    self._i_COD = check_return_property('i_COD', cod/chem_MW * self.i_mass)
+                    self._i_COD = check_return_property('i_COD', cod/self.chem_MW * self.i_mass)
             else: self._i_COD = 0.
 
     @property
@@ -552,7 +589,8 @@ class Component(Chemical):
             super().show()
         else:
             info = component_identity(self, pretty=True)
-        info += '\nComponent-specific properties:\n'
+            info += '\n'
+        info += 'Component-specific properties:\n'
         header = '[Others] '
         section = []
         for field in _component_properties:
@@ -597,57 +635,171 @@ class Component(Chemical):
         return missing
 
 
+    # Cannot directly use `Chemical.copy`
     def copy(self, new_ID, **data):
         '''
         Return a new :class:`Component` object with the same settings with
         alternative data set by kwargs.
+
+        Note that aliases will not be copied.
         '''
-        new = self.__class__.__new__(cls=self.__class__, ID=new_ID)
-        new = copy_attr(new, self, skip=('_ID', '_CAS', '_N_solutes', '_locked_state'))
-        new._ID = new_ID
+        new = self.__class__.__new__(cls=self.__class__, ID=new_ID, search_db=False)
+        skip = ('_ID', '_N_solutes', '_locked_state', '_CAS', '_aliases',)
+        new = copy_attr(new, self, skip=skip)
+        CAS = self.CAS
+        if CAS != self.ID: # for those without actual CAS, it'll be the same as ID
+            num_list = CAS.split('-')
+            if len(num_list) == 3:
+                if (i.isnumeric for i in num_list):
+                    # Only copy CAS when it's an actual CAS (three hyphen-divided parts)
+                    if new.CAS != CAS: new._CAS = CAS
+        if hasattr(self, '_N_solutes'): new.N_solutes = self.N_solutes
 
         phase = data.get('phase') or self._locked_state
         new._locked_state = phase
 
-        new._init_energies(new.Cn, new.Hvap, new.Psat, new.Hfus, new.Sfus,
-                           new.Tm, new.Tb, new.eos, new.phase_ref)
-        new._label_handles()
+        TDependentProperty.RAISE_PROPERTY_CALCULATION_ERROR = False
+        try: 
+            new._init_energies(new.Cn, new.Hvap, new.Psat, new.Hfus, new.Sfus,
+                               new.Tm, new.Tb, new.eos, new.phase_ref, new.S0)
+        except: #!!! this should be needed when biosteam is update appropriately
+            new._init_energies(new.Cn, new.Hvap, new.Psat, new.Hfus, new.Sfus,
+                               new.Tm, new.Tb, new.eos, new.phase_ref)
+        TDependentProperty.RAISE_PROPERTY_CALCULATION_ERROR = True
 
         for i,j in data.items():
-            if i == 'phase':
-                continue
+            if i == 'phase': continue
             setattr(new, i , j)
         return new
 
     __copy__ = copy
 
 
+    # # This won't work if a customized chemical is used
+    # @classmethod
+    # def from_chemical(cls, ID, chemical=None, **data):
+    #     '''Return a new :class:`Component` from a :class:`thermosteam.Chemical` object.'''
+    #     chemical_ID = chemical if isinstance(chemical, str) else chemical.ID
+    #     new = Component(ID=ID, search_ID=chemical_ID, **data)
+    #     return new
+
     @classmethod
-    def from_chemical(cls, ID, chemical=None, phase=None, measured_as=None,
+    def from_chemical(cls, ID, chemical=None, formula=None, phase=None, measured_as=None,
                       i_C=None, i_N=None, i_P=None, i_K=None, i_Mg=None, i_Ca=None,
                       i_mass=None, i_charge=None, i_COD=None, i_NOD=None,
                       f_BOD5_COD=None, f_uBOD_COD=None, f_Vmass_Totmass=None,
                       description=None, particle_size=None, degradability=None,
                       organic=None, **data):
-        '''Return a new :class:`Component` from a :class:`thermosteam.Chemical` object.'''
+        '''
+        Return a new :class:`Component` from a :class:`thermosteam.Chemical` object.
+
+        .. note::
+
+            If you don't have a pre-constructed chemical, you are recommend to use
+            the kwargs `ID`, or `search_ID` in :class:`Component` to search the database
+            instead of using this :func:`Component.from_chemical`.
+
+            E.g., do
+
+                ``S_O = Component(ID='S_O', search_ID='O2', ...)``
+
+            instead of
+
+                ``S_O = Component.from_chemical(ID='S_O', chemical='O2', ...)``
+
+        Examples
+        --------
+        >>> from qsdsan import Component
+        >>> Struvite = Component.from_chemical('Struvite',
+        ...                                    chemical='MagnesiumAmmoniumPhosphate',
+        ...                                    formula='NH4MgPO4·H12O6',
+        ...                                    phase='l', particle_size='Particulate',
+        ...                                    degradability='Undegradable', organic=False)
+        >>> Struvite.show(chemical_info=True)
+        Component: Struvite (phase_ref='l') at phase='l'
+        [Names]  CAS: 7785-21-9
+                 InChI: Mg.H3N.H3O4P/c;;1-5(...
+                 InChI_key: MXZRMHIULZDAKC-U...
+                 common_name: 7785-21-9
+                 iupac_name: ('azanium;magne...
+                 pubchemid: 1.7873e+05
+                 smiles: [NH4+].[O-]P(=O)([O...
+                 formula: NH4MgPO4·H12O6
+        [Groups] Dortmund: <Empty>
+                 UNIFAC: <Empty>
+                 PSRK: <Empty>
+                 NIST: <Empty>
+        [Data]   MW: 137.31 g/mol
+                 Tm: None
+                 Tb: None
+                 Tt: None
+                 Tc: None
+                 Pt: None
+                 Pc: None
+                 Vc: None
+                 Hf: None
+                 S0: 0 J/K/mol
+                 LHV: None
+                 HHV: None
+                 Hfus: 0 J/mol
+                 Sfus: None
+                 omega: None
+                 dipole: None
+                 similarity_variable: 0.080108
+                 iscyclic_aliphatic: 0
+                 combustion: None
+        Component-specific properties:
+        [Others] measured_as: None
+                 description: None
+                 particle_size: Particulate
+                 degradability: Undegradable
+                 organic: False
+                 i_C: 0 g C/g
+                 i_N: 0.057076 g N/g
+                 i_P: 0.12621 g P/g
+                 i_K: 0 g K/g
+                 i_Mg: 0.09904 g Mg/g
+                 i_Ca: 0 g Ca/g
+                 i_mass: 1 g mass/g
+                 i_charge: 0 mol +/g
+                 i_COD: 0 g COD/g
+                 i_NOD: 0 g NOD/g
+                 f_BOD5_COD: 0
+                 f_uBOD_COD: 0
+                 f_Vmass_Totmass: 0
+                 chem_MW: 245.41
+        '''
         new = cls.__new__(cls, ID=ID, phase=phase)
 
-        if chemical is None:
-            chemical = ID
+        if chemical is None: chemical = ID
 
         if isinstance(chemical, str):
-            chemical = Chemical(chemical)
+            chemical = Chemical(chemical, **data)
 
         for field in chemical.__slots__:
             value = getattr(chemical, field, None)
             setattr(new, field, copy_maybe(value))
+
         new._ID = ID
+        if formula and formula != chemical.formula:
+            new._formula = formula
+            if new._Hf is None:
+                new._chem_MW = molecular_weight(new.atoms)
+            else:
+                new.reset_combustion_data()
+        else:
+            new._chem_MW = molecular_weight(new.atoms)
+        if phase: new.at_state(phase)
 
-        if phase: new._locked_state = phase
+        TDependentProperty.RAISE_PROPERTY_CALCULATION_ERROR = False
+        try: 
+            new._init_energies(new.Cn, new.Hvap, new.Psat, new.Hfus, new.Sfus,
+                               new.Tm, new.Tb, new.eos, new.phase_ref, new.S0)
+        except: #!!! this should be needed when biosteam is update appropriately
+            new._init_energies(new.Cn, new.Hvap, new.Psat, new.Hfus, new.Sfus,
+                               new.Tm, new.Tb, new.eos, new.phase_ref)
+        TDependentProperty.RAISE_PROPERTY_CALCULATION_ERROR = True
 
-        new._init_energies(new.Cn, new.Hvap, new.Psat, new.Hfus, new.Sfus,
-                           new.Tm, new.Tb, new.eos, new.phase_ref)
-        new._label_handles()
         new._measured_as = measured_as
         new.i_mass = i_mass
         new.i_C = i_C
@@ -666,8 +818,4 @@ class Component(Chemical):
         new._organic = organic
         new.i_COD = i_COD
         new.i_NOD = i_NOD
-        for i,j in data.items():
-            if i == 'formula':
-                new._formula = j
-            else: setattr(new, i , j)
         return new
\ No newline at end of file
