@@ -5,7 +5,7 @@
 QSDsan: Quantitative Sustainable Design for sanitation and resource recovery systems
 
 This module is developed by:
-    Yalin Li <zoe.yalin.li@gmail.com>
+    Yalin Li <mailto.yalin.li@gmail.com>
 
 This module is under the University of Illinois/NCSA Open Source License.
 Please refer to https://github.com/QSD-Group/QSDsan/blob/main/LICENSE.txt
@@ -19,7 +19,7 @@ import sys
 import pandas as pd
 from warnings import warn
 from thermosteam.utils import registered
-from . import currency, SanStream, WasteStream, ImpactIndicator
+from . import currency, CHECK_IMPACT_ITEM_CONSISTENCY, SanStream, WasteStream, ImpactIndicator
 from .utils import (
     auom, parse_unit, copy_attr,
     format_number as f_num
@@ -28,16 +28,22 @@ from .utils import (
 
 __all__ = ('ImpactItem', 'StreamImpactItem')
 
-def check_source(item, return_item=False, raise_error=True):
-    if item.source:
-        if raise_error:
-            raise ValueError(f'This impact item is copied from {item.source.ID}, '
-                             'value cannot be set.')
-        else:
-            item = item.source
 
-    if return_item:
-        return item
+def raise_inconsistency_error(ID, attr, exist_val, new_val):
+    warn(f'The impact item "{ID}" already exists '
+         f'and has a different {attr} ({exist_val}) '
+         f'than the provided one ({new_val}).')
+
+
+def check_source(item, return_item=False, raise_error=True):
+    if hasattr(item, '_source'):
+        if item.source:
+            if raise_error:
+                raise ValueError(f'This impact item is copied from {item.source.ID}, '
+                                 'value cannot be set.')
+            else: item = item.source
+    if return_item: return item
+
 
 @registered(ticket_name='item')
 class ImpactItem:
@@ -52,13 +58,11 @@ class ImpactItem:
         Functional unit of the impact item.
     price : float
         Price of the item per functional unit.
-    price_unit : str
-        Unit of the price.
     source : :class:`ImpactItem`
         If provided, all attributions and properties of this impact item will
         be copied from the provided source.
     indicator_CFs : kwargs
-        Impact indicators and their characteriziation factors (CFs),
+        Impact indicators and their characterization factors (CFs),
         can be in the form of str=float or str=(float, unit).
 
     Tip
@@ -147,42 +151,87 @@ class ImpactItem:
      'Steel3': <ImpactItem: Steel3>,
      'Steel2': <ImpactItem: Steel2>}
     >>> qs.ImpactItem.clear_registry()
-    All impact items have been removed from registry.
+    All impact items have been removed from the registry.
     >>> qs.ImpactItem.get_all_items()
     {}
+    >>> # Clear all registries for testing purpose
+    >>> from qsdsan.utils import clear_lca_registries
+    >>> clear_lca_registries()
     '''
 
     _items = {}
 
     __slots__ = ('_ID', '_functional_unit', '_price', '_CFs', '_source')
 
-    def __init__(self, ID='', functional_unit='kg', price=0., price_unit='',
-                 source=None, **indicator_CFs):
+    def _format_CF_vals(self, **indicator_CFs):
+        CF_dct = {}
+        for indicator, value in indicator_CFs.items():
+            try: # unit provided for CF
+                CF_value, CF_unit = value
+                CF_dct[indicator] = CF_value, CF_unit
+            except Exception as e: # no unit
+                if 'unpack' in str(sys.exc_info()[1]):
+                    CF_value, CF_unit = value, ''
+                    CF_dct[indicator] = CF_value, CF_unit
+                else:
+                    raise e
+        return CF_dct
 
-        self._register(ID)
+    def _register_with_consistency_check(self, ID, functional_unit, price, source, CF_dct):
+        '''Check consistency in attr values.'''
+        exist_item = ImpactItem.get_item(ID)
+        if exist_item and CHECK_IMPACT_ITEM_CONSISTENCY:
+            self._ID = ID
+            val_dct = {
+                'functional_unit': functional_unit,
+                'price': price,
+                'source': source,
+                }
+            for attr, new_val in val_dct.items():
+                exist_val = getattr(exist_item, attr)
+                if exist_val != new_val and str(exist_val) != str(new_val): # functional_unit as auom
+                    raise_inconsistency_error(ID, attr, exist_val, new_val)
 
-        if source:
-            self.source = source
+            for indicator, (value, unit) in CF_dct.items():
+                indicator_ID = ImpactIndicator.get_indicator(indicator).ID
+                exist_CF = exist_item.CFs.get(indicator_ID)
+                if exist_CF: # indicator already added
+                    if exist_CF != value:
+                        raise_inconsistency_error(ID, 'CF value', exist_CF, value)
+                else: # add the CF value for the new indicator
+                    exist_item.add_indicator(indicator, value, unit)
+            for slot in ImpactItem.__slots__:
+                if slot[1:] in val_dct: continue
+                setattr(self, slot, getattr(exist_item, slot))
+            return exist_item
+        else:
+            self._register(ID)
+            return self
+
+    def __init__(self, ID='', functional_unit='kg', price=0.,
+                 source=None, **indicator_CFs):
+        CF_dct = self._format_CF_vals(**indicator_CFs)
+
+        returned = self._register_with_consistency_check(
+            ID, functional_unit, price, source, CF_dct
+            )
+        if returned is not self: return
+
+        if source: self.source = source
         else:
-            self._source = None
             self._functional_unit = auom(functional_unit)
-            self._update_price(price, price_unit)
+            self._source = None
             self._CFs = {}
-            for indicator, value in indicator_CFs.items():
-                try:
-                    CF_value, CF_unit = value # unit provided for CF
-                    self.add_indicator(indicator, CF_value, CF_unit)
-                except Exception as e:
-                    if 'unpack' in str(sys.exc_info()[1]):
-                        self.add_indicator(indicator, value)
-                    else:
-                        raise e
+            self.price = price
+            for indicator, (value, unit) in CF_dct.items():
+                self.add_indicator(indicator, value, unit)
 
 
     # This makes sure it won't be shown as memory location of the object
     def __repr__(self):
         return f'<ImpactItem: {self.ID}>'
 
+
     def show(self):
         '''Show basic information of this impact item.'''
         info = f'ImpactItem      : {self.ID} [per {self.functional_unit}]'
@@ -203,19 +252,15 @@ class ImpactItem:
 
     _ipython_display_ = show
 
-    def _update_price(self, price=0., unit=''):
-        source_item = check_source(self, True)
-        if not unit or unit == currency:
-            source_item._price = float(price)
-        else:
-            converted = auom(unit).convert(float(price), currency)
-            source_item._price = converted
 
     def add_indicator(self, indicator, CF_value, CF_unit=''):
-        '''Add an indicator with charactorization factor values.'''
+        '''Add an indicator with characterization factor values.'''
+        if not hasattr(self, '_source'): self._source = None
         source_item = check_source(self, True)
         if isinstance(indicator, str):
-            indicator = ImpactIndicator.get_indicator(indicator)
+            ind = ImpactIndicator.get_indicator(indicator)
+            if ind is None: raise RuntimeError(f'The ImpactIndicator "{indicator}" has not been defined.')
+            indicator = ind
 
         CF_unit2 = CF_unit.replace(' eq', '-eq')
 
@@ -225,8 +270,9 @@ class ImpactItem:
                     convert(CF_value, indicator._ureg_unit.units)
             except:
                 raise ValueError(f'Conversion of the given unit {CF_unit} to '
-                                  f'the defaut unit {indicator.unit} is not supported.')
+                                  f'the default unit {indicator.unit} is not supported.')
 
+        if not hasattr(source_item, '_CFs'): source_item._CFs = {}
         source_item._CFs[indicator.ID] = CF_value
 
     def remove_indicator(self, indicator):
@@ -253,9 +299,31 @@ class ImpactItem:
             ID of the new impact item.
         set_as_source : bool
             Whether to set the original impact item as the source.
-        '''
 
-        new = ImpactItem.__new__(ImpactItem)
+        Examples
+        --------
+        >>> import qsdsan as qs
+        >>> GWP = qs.ImpactIndicator('GlobalWarming', alias='GWP', unit='kg CO2-eq')
+        >>> Steel = qs.ImpactItem('Steel', 'kg', GWP=2.55)
+        >>> Steel.show()
+        ImpactItem      : Steel [per kg]
+        Price           : None USD
+        ImpactIndicators:
+                                   Characterization factors
+        GlobalWarming (kg CO2-eq)                      2.55
+        >>> Steel_cp = Steel.copy(set_as_source=True)
+        >>> Steel_cp.show()
+        ImpactItem      : item1 [per kg]
+        Source          : Steel
+        Price           : None USD
+        ImpactIndicators:
+                                   Characterization factors
+        GlobalWarming (kg CO2-eq)                      2.55
+        >>> Steel_cp.source is Steel
+        True
+        '''
+        cls = self.__class__
+        new = cls.__new__(cls)
         new.__init__(new_ID)
 
         if set_as_source:
@@ -288,7 +356,7 @@ class ImpactItem:
         '''Remove all existing impact items from the registry.'''
         cls.registry.clear()
         if print_msg:
-            print('All impact items have been removed from registry.')
+            print('All impact items have been removed from the registry.')
 
 
     @classmethod
@@ -408,7 +476,10 @@ class ImpactItem:
     @property
     def functional_unit(self):
         '''[str] Functional unit of this item.'''
-        return check_source(self, True, False)._functional_unit.units
+        source_item = check_source(self, True, False)
+        if not hasattr(source_item, '_functional_unit'):
+            source_item._functional_unit = auom('kg')
+        return source_item._functional_unit.units
     @functional_unit.setter
     def functional_unit(self, i):
         check_source(self, True)._functional_unit = auom(i)
@@ -422,18 +493,19 @@ class ImpactItem:
     @property
     def price(self):
         '''Price of this item per functional unit.'''
-        return check_source(self, True, False)._price
+        source_item = check_source(self, True, False)
+        if not hasattr(source_item, '_price'): source_item._price = 0.
+        return source_item._price
     @price.setter
-    def price(self, price, unit=''):
-        check_source(self, True)._update_price(price, unit)
+    def price(self, price):
+        source_item = check_source(self, True)
+        source_item._price = float(price)
 
     @property
     def CFs(self):
         '''[dict] Characterization factors of this item for different impact indicators.'''
-        if self.source:
-            return self.source._CFs.copy()
-        else:
-            return self._CFs
+        if self.source: return self.source._CFs.copy()
+        else: return self._CFs
     @CFs.setter
     def CFs(self, indicator, CF_value, CF_unit=''):
         check_source(self, True).add_indicator(indicator, CF_value, CF_unit)
@@ -463,7 +535,7 @@ class StreamImpactItem(ImpactItem):
         If provided, all attributions and properties of this
         :class:`StreamImpactItem` will be copied from the provided source.
     indicator_CFs : kwargs
-        ImpactIndicators and their characteriziation factors (CFs).
+        ImpactIndicators and their characterization factors (CFs).
 
     Tip
     ---
@@ -475,7 +547,7 @@ class StreamImpactItem(ImpactItem):
     Refer to :class:`ImpactItem` for general features.
     Below is about the additional features for :class:`StreamImpactItem`.
 
-    Assume we want to account for the globalwarming potential for methane:
+    Assume we want to account for the global warming potential for methane:
 
     >>> # Make impact indicators
     >>> import qsdsan as qs
@@ -491,7 +563,7 @@ class StreamImpactItem(ImpactItem):
                                Characterization factors
     GlobalWarming (kg CO2-eq)                        28
     >>> # Make a stream and link the stream to the impact item
-    >>> cmps = qs.utils.load_example_cmps()
+    >>> cmps = qs.utils.create_example_components()
     >>> qs.set_thermo(cmps)
     >>> methane = qs.SanStream('methane', Methane=1, units='kg/hr',
     ...                        stream_impact_item=methane_item)
@@ -539,9 +611,16 @@ class StreamImpactItem(ImpactItem):
     __slots__ = ('_ID', '_linked_stream', '_functional_unit', '_CFs', '_source')
 
     def __init__(self, ID='', linked_stream=None, source=None, **indicator_CFs):
-
         self._linked_stream = None
         self.linked_stream = linked_stream
+
+        CF_dct = self._format_CF_vals(**indicator_CFs)
+
+        price = linked_stream.price if linked_stream else 0.
+        returned = self._register_with_consistency_check(
+            ID, 'kg', price, source, CF_dct
+            )
+        if returned is not self: return
         self._register(ID)
 
         if not ID and linked_stream:
@@ -554,15 +633,8 @@ class StreamImpactItem(ImpactItem):
             self._source = None
             self._functional_unit = auom('kg')
             self._CFs = {}
-            for CF, value in indicator_CFs.items():
-                try:
-                    CF_value, CF_unit = value # unit provided for CF
-                    self.add_indicator(CF, CF_value, CF_unit)
-                except Exception as e:
-                    if 'unpack' in str(sys.exc_info()[1]):
-                        self.add_indicator(CF, value)
-                    else:
-                        raise e
+            for indicator, (value, unit) in CF_dct.items():
+                self.add_indicator(indicator, value, unit)
 
 
     def __repr__(self):
@@ -609,8 +681,8 @@ class StreamImpactItem(ImpactItem):
         set_as_source : bool
             Whether to set the original impact item as the source.
         '''
-
-        new = StreamImpactItem.__new__(StreamImpactItem)
+        cls = self.__class__
+        new = cls.__new__(cls)
         new.ID = new_ID
         new._linked_stream = None # initiate this attribute
 
@@ -685,14 +757,17 @@ class StreamImpactItem(ImpactItem):
         if self._linked_stream:
             old_s = self._linked_stream
             self._linked_stream.stream_impact_item = None
-            warn(f'`StreamImpactItem` {self.ID} is unlinked from {old_s.ID} and ' \
-                 f'linked to {new_s.ID}.', stacklevel=2)
+            if old_s is not new_s:
+                warn(f'`StreamImpactItem` {self.ID} is unlinked from {old_s.ID} and ' \
+                     f'linked to {new_s.ID}.')
         if new_s:
-            if hasattr(self, '_ID'):
-                if new_s.stream_impact_item and new_s.stream_impact_item.ID != self.ID:
-                    msg = f'The original `StreamImpactItem` linked to stream {new_s} ' \
-                        f'is replaced with {self}.'
-                    warn(message=msg, stacklevel=2)
+            if new_s.stream_impact_item and new_s.stream_impact_item is not self:
+                if hasattr(self, '_ID'):
+                    warn(f'The original `StreamImpactItem` linked to stream {new_s.ID} '
+                         f'is replaced with {self.ID}.')
+                else:
+                    warn(f'The original `StreamImpactItem` linked to stream {new_s.ID} '
+                         f'is replaced with upon the creation of a new stream.')
             new_s._stream_impact_item = self
         self._linked_stream = new_s
 
@@ -718,4 +793,5 @@ class StreamImpactItem(ImpactItem):
         if self.linked_stream:
             return self.linked_stream.price
         else:
-            return 0.
\ No newline at end of file
+            return 0.
+    _price = price
\ No newline at end of file
