@@ -5,7 +5,7 @@
 QSDsan: Quantitative Sustainable Design for sanitation and resource recovery systems
 
 This module is developed by:
-    Yalin Li <zoe.yalin.li@gmail.com>
+    Yalin Li <mailto.yalin.li@gmail.com>
 
 This module is under the University of Illinois/NCSA Open Source License.
 Please refer to https://github.com/QSD-Group/QSDsan/blob/main/LICENSE.txt
@@ -24,6 +24,7 @@ from warnings import warn
 from . import ImpactIndicator, ImpactItem, Stream, SanStream, SanUnit
 from .utils import (
     auom,
+    clear_lca_registries,
     format_number as f_num
     )
 
@@ -42,7 +43,7 @@ class LCA:
         Lifetime of the LCA.
     lifetime_unit : str
         Unit of lifetime.
-    indicators : iterable
+    indicators : Iterable(obj)
         `ImpactIndicator` objects or their IDs/aliases.
     uptime_ratio : float
         Fraction of time that the system is operating.
@@ -63,11 +64,9 @@ class LCA:
     A system should be constructed prior to LCA, here we import a pre-constructed one.
 
     >>> import qsdsan as qs
-    >>> from qsdsan.utils import load_example_cmps, load_example_sys
-    >>> cmps = load_example_cmps()
-    >>> sys = load_example_sys(cmps)
-    >>> # Uncomment the line below to see the system diagram
-    >>> # sys.diagram()
+    >>> from qsdsan.utils import create_example_system
+    >>> sys = create_example_system()
+    >>> sys.diagram() # doctest: +SKIP
     >>> sys.simulate()
     >>> sys.show()
     System: sys
@@ -151,7 +150,7 @@ class LCA:
     >>> methanol_item = qs.StreamImpactItem(linked_stream=methanol, GWP=2, FEC=13)
     >>> ethanol_item = qs.StreamImpactItem(linked_stream=ethanol, GWP=2.1, FEC=25)
     >>> alcohols_item = qs.StreamImpactItem(linked_stream=alcohols, GWP=-0.2, FEC=-5)
-    >>> brine_item = qs.StreamImpactItem(linked_stream=methanol, GWP=2, FEC=3)
+    >>> brine_item = qs.StreamImpactItem(linked_stream=waste_brine, GWP=2, FEC=3)
 
     Finally, there might be other impacts we want to include in the LCA,
     for example, the electricity needed to operate the system.
@@ -168,19 +167,41 @@ class LCA:
 
     Now we can look at the total impacts associate with this system.
 
-    >>> lca.show() # doctest: +SKIP
+    >>> lca.show() # doctest: +ELLIPSIS
     LCA: sys (lifetime 10 yr)
-    Impacts:
-                                  Construction  Transportation       Stream   Others    Total
-    FossilEnergyConsumption (MJ)       6.5e+03        1.12e+05      1.4e+07 1.95e+06 1.61e+07
-    GlobalWarming (kg CO2-eq)              500        3.75e+04     4.82e+06 8.94e+04 4.95e+06
+    ...
     >>> # Retrieve impacts associated with a specific indicator
-
-    Or breakdowns of the different category
-
-    >>> lca.total_impacts['GlobalWarming'] # doctest: +ELLIPSIS
-    4944207...
+    >>> lca.get_total_impacts()[GWP.ID] # doctest: +ELLIPSIS
+    349737807.9765445...
+    >>> # Or breakdowns of the different category
     >>> lca.get_impact_table('Construction') # doctest: +SKIP
+    >>> # Below is for testing purpose, you do not need it
+    >>> lca.get_impact_table('Construction').to_dict() # doctest: +ELLIPSIS
+    {'Quantity': ...
+    >>> lca.get_impact_table('Transportation').to_dict() # doctest: +ELLIPSIS
+    {'Quantity': ...
+    >>> lca.get_impact_table('Stream').to_dict() # doctest: +ELLIPSIS
+    {'Mass [kg]': ...
+    >>> lca.get_impact_table('Construction').to_dict() # doctest: +ELLIPSIS
+    {'Quantity': ...
+
+    You can also allocate the impact based on mass, energy, value, or a ratio you like
+
+    >>> lca.get_allocated_impacts(sys.products, allocate_by='mass')['waste_brine']['FossilEnergyConsumption'] # doctest: +ELLIPSIS
+    46018518.870...
+    >>> lca.get_allocated_impacts(sys.products, allocate_by='energy')['alcohols']['GlobalWarming'] # doctest: +ELLIPSIS
+    11063009.556...
+    >>> alcohols.price = 5
+    >>> waste_brine.price = 1
+    >>> GWP_alcohols = lca.get_allocated_impacts(sys.products, allocate_by='value')['alcohols']['GlobalWarming']
+    >>> GWP_brine = lca.get_allocated_impacts(sys.products, allocate_by='value')['waste_brine']['GlobalWarming']
+    >>> GWP_alcohols + GWP_brine # doctest: +ELLIPSIS
+    5469807.9765...
+    >>> lca.get_total_impacts(exclude=sys.products)['GlobalWarming'] # doctest: +ELLIPSIS
+    5469807.9765...
+    >>> # Clear all registries for testing purpose
+    >>> from qsdsan.utils import clear_lca_registries
+    >>> clear_lca_registries()
 
     See Also
     --------
@@ -220,6 +241,7 @@ class LCA:
                     raise e
             self.add_other_item(item, f_quantity, unit)
 
+    clear_lca_registries = clear_lca_registries
 
     def _update_system(self, system):
         for u in system.units:
@@ -373,11 +395,12 @@ class LCA:
         Return all stream-related impacts for the given streams,
         normalized to a certain time frame.
         '''
+        isa = isinstance
         if stream_items == None:
             stream_items = self.stream_inventory
-        if not isinstance(stream_items, Iterable):
+        if not isa(stream_items, Iterable):
             stream_items = (stream_items,)
-        if not isinstance(exclude, Iterable):
+        if not isa(exclude, Iterable):
             exclude = (exclude,)
         impacts = dict.fromkeys((i.ID for i in self.indicators), 0.)
         if not time:
@@ -386,8 +409,8 @@ class LCA:
             time = auom(time_unit).convert(float(time), 'hr')
         for j in stream_items:
             # In case that ws instead of the item is given
-            if isinstance(j, Stream):
-                if not isinstance(j, SanStream):
+            if isa(j, Stream):
+                if not isa(j, SanStream):
                     continue
                 ws = j
                 if j.stream_impact_item:
@@ -399,21 +422,21 @@ class LCA:
             if ws in exclude: continue
 
             for m, n in j.CFs.items():
-                if kind == 'all':
+                if kind in ('all', 'total', 'net'):
                     pass
-                elif kind == 'direct_emission':
+                elif kind in ('direct', 'direct_emission'):
                     n = max(n, 0)
                 elif kind == 'offset':
                     n = min(n, 0)
                 else:
                     raise ValueError('kind can only be "all", "direct_emission", or "offset", '
-                                     f'not {kind}.')
+                                     f'not "{kind}".')
                 if m not in impacts.keys():
                     continue
                 impacts[m] += n*time*ws.F_mass
         return impacts
 
-    def get_other_impacts(self):
+    def get_other_impacts(self, time=None, time_unit='hr'):
         '''
         Return all additional impacts from "other" :class:`ImpactItems` objects,
         based on defined quantity.
@@ -421,23 +444,29 @@ class LCA:
         self.refresh_other_items()
         impacts = dict.fromkeys((i.ID for i in self.indicators), 0.)
         other_dct = self.other_items
+        if not time:
+            time = self.lifetime_hr
+        else:
+            time = auom(time_unit).convert(float(time), 'hr')
+        factor = time / self.lifetime_hr
         for i in other_dct.keys():
             item = ImpactItem.get_item(i)
             for m, n in item.CFs.items():
                 if m not in impacts.keys():
                     continue
-                impacts[m] += n*other_dct[i]['quantity']
+                impacts[m] += n*other_dct[i]['quantity']*factor
         return impacts
 
     def get_total_impacts(self, exclude=None, time=None, time_unit='hr'):
         '''Return total impacts, normalized to a certain time frame.'''
         impacts = dict.fromkeys((i.ID for i in self.indicators), 0.)
+        constr = self.get_construction_impacts(self.construction_units, time=time, time_unit=time_unit)
+        trans = self.get_transportation_impacts(self.transportation_units, time=time, time_unit=time_unit)
         ws_impacts = self.get_stream_impacts(stream_items=self.stream_inventory,
                                              exclude=exclude, time=time, time_unit=time_unit)
-        for i in (self.total_construction_impacts,
-                  self.total_transportation_impacts,
-                  ws_impacts,
-                  self.total_other_impacts):
+        other = self.get_other_impacts(time=time, time_unit=time_unit)
+
+        for i in (constr, trans, ws_impacts, other):
             for m, n in i.items():
                 if m not in impacts.keys():
                     continue
@@ -448,22 +477,26 @@ class LCA:
         '''
         Allocate total impacts to one or multiple streams.
 
+        Note that original impacts assigned to the streams will be excluded,
+        i.e., the total impact for allocation will be calculated using
+        `LCA.get_total_impacts(exclude=streams)`.
+
         Parameters
         ----------
-        streams : :class:`WasteStream` or iterable
-            One or a iterable of streams. Note that impacts of these streams will be
+        streams : Iterable(obj)
+            One or a Iterable of streams. Note that impacts of these streams will be
             excluded in calculating the total impacts.
-        allocate_by : str, iterable, or function to generate an iterable
+        allocate_by : str, Iterable, or function to generate an Iterable
             If provided as a str, can be "mass" (`F_mass`), "energy" (`HHV`),
             or 'value' (`F_mass`*`price`) to allocate the impacts accordingly.
-            If provided as an iterable (no need to normalize so that sum of the iterable is 1),
-            will allocate impacts according to the iterable.
+            If provided as an Iterable (no need to normalize so that sum of the Iterable is 1),
+            will allocate impacts according to the Iterable.
             If provided as a function,  will call the function to generate an
-            iterable to allocate the impacts accordingly.
+            Iterable to allocate the impacts accordingly.
 
         .. note::
 
-            Energy of the stream will be calcuated as the sum of HHVs of all components
+            Energy of the stream will be calculated as the sum of HHVs of all components
             in the stream.
 
         '''
@@ -488,7 +521,8 @@ class LCA:
             ratios = allocate_by()
         else:
             raise ValueError('allocate_by can only be "mass", "energy", "value", '
-                             'an iterable, or a function to generate an iterable.')
+                             'an Iterable (with the same length as `streams`), '
+                             'or a function to generate an Iterable.')
         if ratios.sum() == 0:
             raise ValueError('Calculated allocation ratios are all zero, cannot allocate.')
         ratios = ratios/ratios.sum()
@@ -497,8 +531,7 @@ class LCA:
                 continue
             if not s in self.system.streams:
                 raise ValueError(f'`WasteStream` {s} not in the system.')
-            allocated[s.ID] = dict.fromkeys(impact_dct.keys(),
-                                            (ratios[n]*impact_vals).sum())
+            allocated[s.ID] = dict(zip(impact_dct.keys(), ratios[n]*impact_vals))
         return allocated
 
 
@@ -755,7 +788,7 @@ class LCA:
 
     @property
     def construction_units(self):
-        '''[set] All units in the linked system with constrution activity.'''
+        '''[set] All units in the linked system with construction activity.'''
         return self._construction_units
 
     @property
