@@ -5,7 +5,7 @@
 QSDsan: Quantitative Sustainable Design for sanitation and resource recovery systems
 
 This module is developed by:
-    Yalin Li <zoe.yalin.li@gmail.com>
+    Yalin Li <mailto.yalin.li@gmail.com>
     Joy Zhang <joycheung1994@gmail.com>
 
 Part of this module is based on the BioSTEAM package:
@@ -22,15 +22,35 @@ for license details.
 import numpy as np
 from collections import defaultdict
 from collections.abc import Iterable
-from matplotlib import pyplot as plt
-from biosteam.utils import Inlets, Outlets, format_title
-from . import currency, Unit, Stream, SanStream, WasteStream, \
-    Construction, Transportation, System
+from warnings import warn
+from biosteam.utils import (
+    Inlets,
+    MissingStream,
+    MockStream,
+    Outlets,
+    Scope,
+    )
+try: from biosteam.utils import TemporaryStream
+except:
+    class TemporaryStream: pass
+from . import (
+    Construction,
+    currency,
+    Equipment,
+    HeatUtility,
+    PowerUtility,
+    SanStream,
+    Stream,
+    System,
+    Transportation,
+    Unit,
+    WasteStream,
+    )
+from .utils import SanUnitScope
 
 __all__ = ('SanUnit',)
 
 def _update_init_with(init_with, ins_or_outs, size):
-
     if isinstance(init_with, str):
         init_with = dict.fromkeys([f'{ins_or_outs}{n}' for n in range(size)], init_with)
         return init_with
@@ -47,11 +67,10 @@ def _update_init_with(init_with, ins_or_outs, size):
         new_init_with = {}
 
     for k, v in init_with.items():
-        if k in ('Else', 'else'):
-            continue
+        if k in ('Else', 'else'): continue
         v_lower = v.lower()
-        if v_lower in ('stream', 's'):
-            new_init_with[k] = 's'
+        if v_lower in ('mockstream', 'multistream', 'stream', 'temporarystream', 's', 'ms', 'ts'): 
+            new_init_with[k] = 's' # biosteam-native streams
         elif v_lower in ('sanstream', 'ss'):
             new_init_with[k] = 'ss'
         elif v_lower in ('wastestream', 'ws'):
@@ -60,13 +79,22 @@ def _update_init_with(init_with, ins_or_outs, size):
             raise ValueError(f'Stream type for {k} is invalid, ' \
                              'valid values are "Stream" or "s", "SanStream" or "ss", ' \
                              'and "WasteStream" or "ws".')
-
     return new_init_with
 
+
+def _replace_missing_streams(port, missing):
+    for idx, s_type in missing:
+        if s_type == 's': continue
+        s = port[idx]
+        stream_class = SanStream if s_type=='ss' else WasteStream
+        port.replace(s, stream_class.from_stream(stream=s))
+
+
 def _get_inf_state(inf):
     inf._init_state()
     return inf._state
 
+
 add2list = lambda lst, item: lst.extend(item) if isinstance(item, Iterable) \
     else lst.append(item)
 
@@ -79,17 +107,19 @@ class SanUnit(Unit, isabstract=True):
     Parameters
     ----------
     init_with : str or dict
-        Which class of stream the :class:`SanUnit` will be initialized with,
+        Which class of stream the :class:`~.SanUnit` will be initialized with,
         can be "Stream" (shorthanded as "s"), "SanStream" ("ss"), or "WasteStream" ("ws").
         When provided as a str, all streams will be of the same class;
         when provided as a dict, use "ins" or "outs" followed with the order number
         (i.e., ins0, outs-1) as keys; you can use ":" to denote a range (e.g., ins2:4);
         you can also use "else" to specify the stream class for non-provided ones.
-    construction : :class:`~.Construction` or iterable
-        Contains construction information.
-    transportation : :class:`~.Transportation` or iterable
-        Contains construction information.
-    add_OPEX : float or dict
+    construction : Iterable(obj)
+        :class:`~.Construction` with construction information.
+    transportation : Iterable(obj)
+        :class:`~.Transportation` with transportation information.
+    equipments: Iterable(obj)
+        :class:`~.Equipment` with equipment information.
+    add_OPEX : float/int or dict
         Operating expense per hour in addition to utility cost (assuming 100% uptime).
         Float input will be automatically converted to a dict with the key being
         "Additional OPEX".
@@ -122,110 +152,167 @@ class SanUnit(Unit, isabstract=True):
             Regardless of `F_BM_default`, design (F_D), pressure (F_P),
             and material (F_M) factors are all defaulted to 1.
 
+    isdynamic : bool
+        If this unit is simulated dynamically with rate equations.
+    exogenous_var : iterable[:class:`ExogenousDynamicVariable`], optional
+        Any exogenously dynamic variables that affect the process mass balance.
+        Must be independent of state variables of the process model (if has one).
+    kwargs : dict
+        Additional keyword arguments that can be set for this unit.
+
     See Also
     --------
     `biosteam.Unit <https://biosteam.readthedocs.io/en/latest/Unit.html>`_
     `thermosteam.Stream <https://thermosteam.readthedocs.io/en/latest/Stream.html>`_
 
     '''
-
-    ticket_name = 'SU'
-
     def __init__(self, ID='', ins=None, outs=(), thermo=None, init_with='WasteStream',
                  construction=(), transportation=(), equipments=(),
                  add_OPEX={}, uptime_ratio=1., lifetime=None, F_BM_default=None,
-                 isdynamic=False, **kwargs):
+                 isdynamic=False, exogenous_vars=(), **kwargs):
+        self._system = None
         self._register(ID)
         self._specification = None
         self._load_thermo(thermo)
         self._init_with = init_with
         self._init_ins(ins, init_with)
         self._init_outs(outs, init_with)
+        #: All heat utilities associated to unit. Cooling and heating requirements 
+        #: are stored here (including auxiliary requirements). The number of heat utilities created is given by the
+        #: class attribute :attr:`~Unit._N_heat_utilities`.
+        self.heat_utilities: tuple[HeatUtility, ...] = tuple([HeatUtility() for i in range(self._N_heat_utilities)])
+        
+        #: Electric utility associated to unit (including auxiliary requirements).
+        self.power_utility: PowerUtility = PowerUtility()
+
         self._init_utils()
         self._init_results()
-        self._init_specification()
-        self._assert_compatible_property_package()
-        self.construction = construction
-        self.transportation = transportation
-        for equip in equipments:
-            equip._linked_unit = self
-        self.equipments = equipments
-        self.add_OPEX = add_OPEX
+        self._init_specifications()
+        #: Whether to prioritize unit operation specification within recycle loop (if any).
+        self.prioritize: bool = False
+
+        if not kwargs.get('skip_property_package_check'):
+            self._assert_compatible_property_package()
+        for i in (*construction, *transportation, *equipments):
+            i._linked_unit = self
+        # Make fresh ones for each unit
+        self.construction = () if not construction else construction
+        self.transportation = () if not transportation else transportation
+        self.equipments = () if not equipments else equipments
+        self.add_OPEX = add_OPEX.copy()
         self.uptime_ratio = 1.
         self.lifetime = lifetime
         if F_BM_default:
             F_BM = self.F_BM
             self.F_BM = defaultdict(lambda: F_BM_default)
             self.F_BM.update(F_BM)
+
+        # For units with different state headers, should update it in the unit's ``__init__``
         self.isdynamic = isdynamic
-        # For units with different states, should update it in the unit's ``__init__``
-        self._state_header = [f'{cmp.ID} [mg/L]' for cmp in self.components] + ['Q [m3/d]']
+        self._exovars = exogenous_vars
         for attr, val in kwargs.items():
             setattr(self, attr, val)
 
+    # For backward compatibility
+    def _init_specifications(self):
+        try: super()._init_specifications() # new biosteam
+        except: self._init_specification() # old biosteam
+
+    def add_specifications(self, function):
+        try: super().add_specifications(self, function) # new biosteam
+        except: self.specification = function # old biosteam
+    
+    @property
+    def run_after_specification(self):
+        try: return self.run_after_specifications # new biosteam
+        except: # old biosteam
+            self._run_after_specification = False
+            return self._run_after_specification
+    @run_after_specification.setter
+    def run_after_specification(self, i):
+        if hasattr(self, 'add_specifications'): self.run_after_specifications = i # new biosteam
+        else: self._run_after_specification = i # old biosteam
+
 
     def _convert_stream(self, strm_inputs, streams, init_with, ins_or_outs):
+        isa = isinstance
         if not streams:
-            return []
-
-        if isinstance(init_with, str):
+            return [], []
+        if isa(init_with, str):
             init_with = dict.fromkeys([f'{ins_or_outs}{n}'
                                        for n in range(len(streams))], init_with)
-
         # Do not change pre-defined stream types
-        if isinstance(strm_inputs, Stream): # input is a stream
+        if isa(strm_inputs, Stream): # input is a stream
             init_with[f'{ins_or_outs}0'] = type(strm_inputs).__name__
-         # input is an iterable of stream
-        elif isinstance(strm_inputs, Iterable) and not isinstance(strm_inputs, str):
-            for n, s in enumerate(strm_inputs):
-                if isinstance(s, Stream):
-                    init_with[f'{ins_or_outs}{n}'] = type(s).__name__
+        # `input` is an iterable of stream
+        elif isa(strm_inputs, Iterable) and not isa(strm_inputs, str):
+            for in_or_out, s in zip(ins_or_outs, strm_inputs):
+                if isa(s, Stream):
+                    init_with[in_or_out] = type(s).__name__
 
         init_with = _update_init_with(init_with, ins_or_outs, len(streams))
 
-        converted = []
+        converted, missing = [], []
         for k, v in init_with.items():
-            if not ins_or_outs in k:
+            if not ins_or_outs in k: # leave out outsX when going through ins and vice versa
                 continue
-
-            num = k.split(ins_or_outs)[-1]
-            s = streams[int(num)]
-            if v == 's':
-                converted.append(s)
+            num = int(k.split(ins_or_outs)[-1])
+            s = streams[num]
+            if isa(s, MissingStream):
+                missing.append((num, v))
+                continue
+            if v == 's': # stream/mockstream/multistream/temporarystream
+                converted.append(s) # no conversion for these types
             elif v == 'ss':
-                converted.append(SanStream.from_stream(SanStream, s))
+                converted.append(SanStream.from_stream(stream=s))
             else:
-                converted.append(WasteStream.from_stream(WasteStream, s))
+                converted.append(WasteStream.from_stream(stream=s))
 
-        diff = len(converted) - len(streams)
+        diff = len(converted) + len(missing) - len(streams)
         if diff != 0:
             raise ValueError(f'Type(s) of {diff} stream(s) has/have not been specified.')
 
-        return converted
+        return converted, missing
 
 
     def _init_dynamic(self):
         self._state = None
         self._dstate = None
-        self._state_header = [f'{cmp.ID} [mg/L]' for cmp in self.components]
+        self._ins_QC = np.zeros((len(self._ins), len(self.components)+1))
+        self._ins_dQC = self._ins_QC.copy()
         self._ODE = None
-        self._mock_dyn_sys = System(self.ID+'_dynmock', path=(self,))
+        self._AE = None
+        if not hasattr(self, '_mock_dyn_sys'):
+            self._mock_dyn_sys = System(self.ID+'_dynmock', path=(self,))
+        if not hasattr(self, '_state_header'):
+            self._state_header = [f'{cmp.ID} [mg/L]' for cmp in self.components] + ['Q [m3/d]']
+        # Shouldn't need to re-create the mock system every time
+        # if hasattr(self, '_mock_dyn_sys'):
+        #     sys = self._mock_dyn_sys
+        #     sys.registry.discard(sys)
+        # self._mock_dyn_sys = System(self.ID+'_dynmock', path=(self,))
 
 
     def _init_ins(self, ins, init_with):
         super()._init_ins(ins)
-        converted = self._convert_stream(ins, self.ins, init_with, 'ins')
-        self._ins = Inlets(self, self._N_ins, converted, self._thermo,
-                           self._ins_size_is_fixed, self._stacklevel)
+        converted, missing = self._convert_stream(ins, self.ins, init_with, 'ins')
+        _ins = self._ins = Inlets(self, self._N_ins, converted, self._thermo,
+                                  self._ins_size_is_fixed, self._stacklevel)
+        # Cannot do it within `_convert_stream` as it creates new streams and
+        # cause error in `Inlets/Outlets._redock`
+        _replace_missing_streams(_ins, missing)
+
 
     def _init_outs(self, outs, init_with):
         super()._init_outs(outs)
-        converted = self._convert_stream(outs, self.outs, init_with, 'outs')
-        self._outs = Outlets(self, self._N_outs, converted, self._thermo,
-                           self._outs_size_is_fixed, self._stacklevel)
+        converted, missing = self._convert_stream(outs, self.outs, init_with, 'outs')
+        _outs = self._outs = Outlets(self, self._N_outs, converted, self._thermo,
+                                     self._outs_size_is_fixed, self._stacklevel)
+        _replace_missing_streams(_outs, missing)
 
     def _init_results(self):
         super()._init_results()
+        self.add_OPEX = {}
 
     def __repr__(self):
         return f'<{type(self).__name__}: {self.ID}>'
@@ -270,27 +357,41 @@ class SanUnit(Unit, isabstract=True):
         info = info.replace('\n ', '\n    ')
         return info[:-1]
 
-    def simulate(self, t_span=(0, 0), start_from_cached_state=True, 
-                 solver='', **kwargs):
+    def set_dynamic_tracker(self, *subjects, **kwargs):
+        """
+        Set up an :class:`SystemScope` object to track the dynamic data.
+
+        Parameters
+        ----------
+        *subjects :
+            Any subjects of the system to track, which must have an `.scope`
+            attribute of type :class:`Scope`.
+        """
+        sys = self._mock_dyn_sys
+        if self.isdynamic:
+            sys._scope = {'subjects':subjects, 'kwargs':kwargs}
+        else:
+            warn(f'{self.ID} is not a dynamic unit, cannot set tracker.')
+
+    def simulate(self, **kwargs):
         '''
         Converge mass and energy flows, design, and cost the unit.
-        
+
         .. note::
-            
-            If this unit is a dynamic unit, ODEs will be run after ``_run``
+
+            If this unit is a dynamic unit, AEs/ODEs will be run after ``_run``
             and/or ``specification``.
-        
+
         Parameters
         ----------
-        t_span : tuple(float, float)
-            Integration time span for dynamic units.
-        start_from_cached_state: bool
-            Whether to start from the cached state.
         kwargs : dict
-            Other keyword arguments that will be passed to ``scipy.integrate.solve_ivp``
-            
+            Keyword arguments that will be passed to ``biosteam.systeam.dynamic_run``
+            (useful when running dynamic simulation).
+
         See Also
         --------
+        :func:`biosteam.System.dynamic_run`
+
         `scipy.integrate.solve_ivp <https://docs.scipy.org/doc/scipy/reference/generated/scipy.integrate.solve_ivp.html>`_
         '''
         super().simulate()
@@ -298,73 +399,48 @@ class SanUnit(Unit, isabstract=True):
             sys = self._mock_dyn_sys
             sys._feeds = self.ins
             sys._products = self.outs
-            sys.simulate(t_span=t_span,
-                         start_from_cached_state=start_from_cached_state,
-                         **kwargs)
+            sys.simulate(**kwargs)
             self._summary()
 
-    def plot_state_over_time(self, system=None, state_var=()):
-        '''
-        Plot the selected state variable over simulated time,
-        all state variables will be plotted if not given.
-        
-        Parameters
-        ----------
-        system : obj
-            The ``System`` object where this unit is simulated,
-            leave as None if this unit is simulated alone.
-        state_var : str of Iterable
-            Name of the state variables, if unsure, check the `_state_header` attribute.
-        '''
-        if not self.isdynamic:
-            raise RuntimeError(f'Unit {self.ID} is not a dynamic unit, '
-                               'cannot plot state over time.')
-        state_var = [state_var] if isinstance(state_var, str) else state_var
-        sys = system or self._mock_dyn_sys
-        try:
-            df = sys._state2df()
-        except:
-            system_or_unit = 'system' if sys==system else 'unit'
-            ID = sys.ID.rstrip('_dynmock')
-            raise RuntimeError(f'The {system_or_unit} {ID} has not been simulated, '
-                               'please simulate first.')    
-        t = df.iloc[:, 0]
-        unit_df = df.xs(self.ID, axis=1)
-        variables = [i.split(' ')[0] for i in self._state_header]
-        units = [i.split(' ')[1] for i in self._state_header]
-        unit_df.columns = variables
-        if state_var:
-            unit_df = unit_df.loc[:, state_var]
-        fig, ax = plt.subplots(figsize=(8, 4.5))
-        for var in unit_df.columns:
-            ax.plot(t, getattr(unit_df, var), '-o',
-                    label=f'{var} {units[variables.index(var)]}')
-        ax.legend(loc='best')
-        ylabel = 'Concentration' if 'Q' not in state_var else 'Concentration or flow'
-        ax.set(xlabel='Time [d]', ylabel=ylabel)
-        return fig, ax
-
     def show(self, T=None, P=None, flow='g/hr', composition=None, N=15, IDs=None, stream_info=True):
         '''Print information of the unit, including waste stream-specific information.'''
         print(self._info(T, P, flow, composition, N, IDs, stream_info))
 
     def add_equipment_design(self):
+        unit_design = self.design_results
+        unit_units = self._units
+        F_BM, F_D, F_P, F_M, lifetime = \
+            self.F_BM, self.F_D, self.F_P, self.F_M, self._default_equipment_lifetime
+        isa = isinstance
+        get = getattr
+        def update_unit_attr(unit_attr, equip_ID, equip_attr):
+            if isa(equip_attr, dict):
+                unit_attr.update(equip_attr)
+            else:
+                unit_attr[equip_ID] = equip_attr
         for equip in self.equipments:
-            name = equip.name or format_title(type(equip).__name__)
-            equip_design = equip._design()
+            equip_ID = equip.ID
+            equip_design = equip._design_results = equip._design()
             equip_design = {} if not equip_design else equip_design
-            self.design_results.update(equip_design)
+            unit_design.update(equip_design)
+
+            equip_units = {} if not equip.units else equip.units
+            unit_units.update(equip_units)
+            for unit_attr, equip_attr in zip(
+                    (F_BM, F_D, F_P, F_M, lifetime),
+                    ('F_BM', 'F_D', 'F_P', 'F_M', 'lifetime'),
+                    ):
+                update_unit_attr(unit_attr, equip_ID, get(equip, equip_attr))
 
-            equip_units = {} if not equip.design_units else equip.design_units
-            self._units.update(equip_units)
-            self.F_BM[name] = equip.F_BM
-            if equip.lifetime:
-                self._default_equipment_lifetime[name] = equip.lifetime
 
     def add_equipment_cost(self):
+        unit_cost = self.baseline_purchase_costs
         for equip in self.equipments:
-            name = equip.name or format_title(type(equip).__name__)
-            self.baseline_purchase_costs[name] = equip._cost()
+            equip_cost = equip._baseline_purchase_costs = equip._cost()
+            if isinstance(equip_cost, dict):
+                unit_cost.update(equip_cost)
+            else:
+                unit_cost[equip.ID] = equip_cost
 
     def add_construction(self, add_unit=True, add_design=True, add_cost=True,
                          add_lifetime=True):
@@ -379,6 +455,7 @@ class SanUnit(Unit, isabstract=True):
             if add_lifetime and i.lifetime:
                 self._default_equipment_lifetime[i.item.ID] = i.lifetime
 
+
     @property
     def components(self):
         '''[Components] The :class:`~.Components` object associated with this unit.'''
@@ -390,34 +467,63 @@ class SanUnit(Unit, isabstract=True):
         return self._isdynamic
     @isdynamic.setter
     def isdynamic(self, i):
-        if hasattr(self, '_isdynamic'):
+        hasfield = hasattr
+        if hasfield(self, '_isdynamic'):
             if self._isdynamic == bool(i):
                 return
-        else:
-            self._isdynamic = bool(i)
+        else: self._isdynamic = bool(i)
+        if self.hasode:
             self._init_dynamic()
+            if hasattr(self, '_mock_dyn_sys'):
+                ID = self.ID+'_dynmock'
+                System.registry.discard(ID)
+            self._mock_dyn_sys = System(self.ID+'_dynmock', path=(self,))
 
+    @property
+    def exo_dynamic_vars(self):
+        '''[iterable[:class:`ExogenousDynamicVariable`]] Exogenous dynamic
+        variables that affect the process mass balance, e.g., temperature,
+        sunlight irradiance.'''
+        return self._exovars
 
-    def _collect_ins_state(self):
-        return np.array([inf._state for inf in self._ins])
-        # states = []
-        # for inf in self.ins:
-        #     state = inf._state
-        #     if state is not None:
-        #         if not isinstance(state, float):
-        #             states.append(state)
-        #         else:
-        #             inf._init_state()
-        #             states.append(inf._state)
-        # return np.array(states)
-                
-            
-    def _collect_ins_dstate(self):
-        return np.array([inf._dstate for inf in self._ins])
+    def eval_exo_dynamic_vars(self, t):
+        '''Evaluates the exogenous dynamic variables at time t.'''
+        return [var(t) for var in self._exovars]
+
+    @property
+    def scope(self):
+        """A tracker of the unit's time-series data during dynamic simulation."""
+        if not hasattr(self, '_scope'):
+            self._scope = SanUnitScope(self)
+        return self._scope
+
+    @scope.setter
+    def scope(self, s):
+        if not isinstance(s, Scope):
+            raise TypeError(f'{s} must be an {Scope} not {type(s)}.')
+        if self is not s.subject:
+            raise ValueError(f'The subject of {s} must be {self} not {s.subject}.')
+        self._scope = s
+
+    @property
+    def hasode(self):
+        """Whether this unit's dynamic states are determined by ordinary differential equations."""
+        return hasattr(self, '_compile_ODE')
+
+    def reset_cache(self, dynamic_system=False):
+        '''Reset cached states for dynamic units.'''
+        super().reset_cache()
+        if self.hasode or dynamic_system:
+            self._init_dynamic()
+            for s in self.outs:
+                s.empty()
+
+    def get_retained_mass(self, biomass_IDs):
+        warn(f'The retained biomass in {self.ID} is ignored.')
 
     @property
     def construction(self):
-        '''[:class:`~.Construction` or iterable] Contains construction information.'''
+        '''Iterable(obj) :class:`~.Construction` with construction information.'''
         return self._construction
     @construction.setter
     def construction(self, i):
@@ -435,7 +541,7 @@ class SanUnit(Unit, isabstract=True):
 
     @property
     def transportation(self):
-        '''[:class:`~.Transportation` or iterable] Contains transportation information.'''
+        '''Iterable(obj) :class:`~.Transportation` with transportation information.'''
         return self._transportation
     @transportation.setter
     def transportation(self, i):
@@ -451,6 +557,25 @@ class SanUnit(Unit, isabstract=True):
                         f'Only `Transportation` can be included, not {type(j).__name__}.')
         self._transportation = i
 
+    @property
+    def equipments(self):
+        '''Iterable(obj) :class:`~.Equipment` with equipment information.'''
+        return self._equipments
+    @equipments.setter
+    def equipments(self, i):
+        isa = isinstance
+        if isa(i, Equipment):
+            i = (i,)
+        else:
+            if not isa(i, Iterable):
+                raise TypeError(
+                    f'Only `Equipment` object  can be included, not {type(i).__name__}.')
+            for j in i:
+                if not isa(j, Equipment):
+                    raise TypeError(
+                        f'Only `Equipment` can be included, not {type(j).__name__}.')
+        self._equipments = i
+
     @property
     def add_OPEX(self):
         '''
@@ -458,13 +583,14 @@ class SanUnit(Unit, isabstract=True):
         Float input will be automatically converted to a dict with the key being
         "Additional OPEX".
         '''
-        return {'Additional OPEX': self._add_OPEX} if isinstance(self._add_OPEX, float) \
+        return {'Additional OPEX': self._add_OPEX} if isinstance(self._add_OPEX, (float, int)) \
             else self._add_OPEX
     @add_OPEX.setter
     def add_OPEX(self, i):
-        if isinstance(i, float):
+        isa = isinstance
+        if isa(i, (float, int)):
             i = {'Additional OPEX': i}
-        if not isinstance(i, dict):
+        if not isa(i, dict):
             raise TypeError(
                 f'add_OPEX can only be float of dict, not {type(i).__name__}.')
         self._add_OPEX = i
@@ -506,8 +632,8 @@ class SanUnit(Unit, isabstract=True):
     @uptime_ratio.setter
     def uptime_ratio(self, i):
         if not 0 <=i<= 1:
-            raise ValueError(f'Uptime must be between 0 and 1 (100%), not {i}.')
-        self._uptime_ratio = float(i)
+            warn(f'`uptime_ratio` of {i:.2f} for unit {self.ID} is not in 0 and 1 (100%).')
+        self._uptime_ratio = i
 
     @property
     def lifetime(self):
