@@ -6,29 +6,41 @@ QSDsan: Quantitative Sustainable Design for sanitation and resource recovery sys
 
 This module is developed by:
     Smiti Mittal <smitimittal@gmail.com>
+    Yalin Li <mailto.yalin.li@gmail.com>
 
 This module is under the University of Illinois/NCSA Open Source License.
 Please refer to https://github.com/QSD-Group/QSDsan/blob/main/LICENSE.txt
 for license details.
 '''
 
-from ._electrode import *
-from ._membrane import *
+from ._aeration import *
 from ._column import *
+from ._electrode import *
+from ._encapsulation import *
 from ._machine import*
-
+from ._membrane import *
+from ._vacuum_pump import *
+from ._vertical_mixer import *
 
 from . import (
-    _electrode,
-    _membrane,
+    _aeration,
     _column,
-    _machine
+    _electrode,
+    _encapsulation,
+    _machine,
+    _membrane,
+    _vacuum_pump,
+    _vertical_mixer,
     )
 
 
 __all__ = (
-    *_electrode.__all__,
-    *_membrane.__all__,
+    *_aeration.__all__,
     *_column.__all__,
+    *_electrode.__all__,
+    *_encapsulation.__all__,
     *_machine.__all__,
+    *_membrane.__all__,
+    *_vacuum_pump.__all__,
+    *_vertical_mixer.__all__,
            )
\ No newline at end of file
