@@ -5,7 +5,8 @@
 QSDsan: Quantitative Sustainable Design for sanitation and resource recovery systems
 
 This module is developed by:
-    Yalin Li <zoe.yalin.li@gmail.com>
+
+    Yalin Li <mailto.yalin.li@gmail.com>
 
 This module is under the University of Illinois/NCSA Open Source License.
 Please refer to https://github.com/QSD-Group/QSDsan/blob/main/LICENSE.txt
@@ -15,15 +16,21 @@ for license details.
 from ._aeration import *
 from ._asm1 import *
 from ._asm2d import *
+from ._adm1 import *
+from ._decay import *
 
 from . import (
     _aeration,
     _asm1,
-    _asm2d
+    _asm2d,
+    _adm1,
+    _decay,
     )
 
 __all__ = (
     *_aeration.__all__,
     *_asm1.__all__,
     *_asm2d.__all__,
-           )
\ No newline at end of file
+    *_adm1.__all__,
+    *_decay.__all__,
+    )
\ No newline at end of file
