@@ -12,17 +12,16 @@ for license details.
 
 from thermosteam.utils import chemicals_user
 from thermosteam import settings
-from qsdsan import Components, Processes, _pk
-from ..utils import ospath, data_path, save_pickle, load_pickle
+from qsdsan import Components, Processes, CompiledProcesses
+from ..utils import ospath, data_path
 
-__all__ = ('load_asm1_cmps', 'ASM1')
+__all__ = ('create_asm1_cmps', 'ASM1')
 
 _path = ospath.join(data_path, 'process_data/_asm1.tsv')
-_path_cmps = ospath.join(data_path, '_asm1_cmps.pckl')
 _load_components = settings.get_default_chemicals
 
 ############# Components with default notation #############
-def _create_asm1_cmps(pickle=False):
+def create_asm1_cmps(set_thermo=True):
     cmps = Components.load_default()
 
     S_I = cmps.S_U_Inf.copy('S_I')
@@ -42,7 +41,7 @@ def _create_asm1_cmps(pickle=False):
 
     X_BH = cmps.X_OHO.copy('X_BH')
     X_BH.description = 'Active heterotrophic biomass'
-    
+
 
     X_BA = cmps.X_AOO.copy('X_BA')
     X_BA.description = 'Active autotrophic biomass'
@@ -60,7 +59,6 @@ def _create_asm1_cmps(pickle=False):
     S_NO.description = 'Nitrate and nitrite nitrogen'
 
     S_NH = cmps.S_NH4.copy('S_NH')
-
     S_ND = cmps.S_F.copy('S_ND')
     S_ND.description = 'Soluble biodegradable organic nitrogen'
     S_ND.measured_as = 'N'
@@ -80,20 +78,11 @@ def _create_asm1_cmps(pickle=False):
                             S_O, S_NO, S_NH, S_ND, X_ND, S_ALK,
                             cmps.S_N2, cmps.H2O])
     cmps_asm1.compile()
+    if set_thermo: settings.set_thermo(cmps_asm1)
 
-    if pickle:
-        save_pickle(cmps_asm1, _path_cmps)
     return cmps_asm1
 
-
-#_create_asm1_cmps(True)
-
-def load_asm1_cmps():
-    if _pk:
-        return load_pickle(_path_cmps)
-    else:
-        return _create_asm1_cmps(pickle=False)
-
+# create_asm1_cmps()
 
 
 ############ Processes in ASM1 #################
@@ -150,7 +139,7 @@ def load_asm1_cmps():
 #     )
 
 @chemicals_user
-class ASM1(Processes):
+class ASM1(CompiledProcesses):
     '''
     Activated Sludge Model No. 1 in original notation. [1]_, [2]_
 
@@ -204,9 +193,19 @@ class ASM1(Processes):
         The default is 0.4.
     k_a : float, optional
         Ammonification rate constant, in [m^3/g COD/d]. The default is 0.05.
+    fr_SS_COD : float, optional
+        Ash content for organic particulate components, in [g SS/g COD]. The default is 0.75.
     path : str, optional
         Alternative file path for the Gujer matrix. The default is None.
 
+    Examples
+    --------
+    >>> from qsdsan import processes as pc, set_thermo
+    >>> cmps = pc.create_asm1_cmps()
+    >>> asm1 = pc.ASM1()
+    >>> asm1.show()
+    ASM1([aero_growth_hetero, anox_growth_hetero, aero_growth_auto, decay_hetero, decay_auto, ammonification, hydrolysis, hydrolysis_N])
+
     References
     ----------
     .. [1] Henze, M.; Gujer, W.; Mino, T.; Loosdrecht, M. van. Activated Sludge
@@ -229,18 +228,19 @@ class ASM1(Processes):
                 k_h=3.0, K_X=0.1, mu_A=0.5, K_NH=1.0, b_A=0.05, K_O_A=0.4, k_a=0.05,
                 fr_SS_COD=0.75, path=None, **kwargs):
         if not path: path = _path
-        
+
         cmps = _load_components(components)
         cmps.X_BH.i_N = cmps.X_BA.i_N = i_XB
         cmps.X_P.i_N = cmps.X_I.i_N = i_XP
         cmps.X_I.i_mass = cmps.X_S.i_mass = cmps.X_P.i_mass = cmps.X_BH.i_mass = cmps.X_BA.i_mass = fr_SS_COD
         cmps.refresh_constants()
-        
+
         self = Processes.load_from_file(path,
                                         conserved_for=('COD', 'charge', 'N'),
                                         parameters=cls._params,
                                         components=cmps,
-                                        compile=True)
+                                        compile=False)
+        self.compile(to_class=cls)
 
         self.set_parameters(Y_A=Y_A, Y_H=Y_H, f_P=f_P, mu_H=mu_H, K_S=K_S, K_O_H=K_O_H,
                             K_NO=K_NO, b_H=b_H, eta_g=eta_g, eta_h=eta_h, k_h=k_h,
