@@ -4,23 +4,32 @@ QSDsan: Quantitative Sustainable Design for sanitation and resource recovery sys
 
 This module is developed by:
     Joy Zhang <joycheung1994@gmail.com>
+    Yalin Li <mailto.yalin.li@gmail.com>    
 
 This module is under the University of Illinois/NCSA Open Source License.
 Please refer to https://github.com/QSD-Group/QSDsan/blob/main/LICENSE.txt
 for license details.
 '''
 
-from .. import SanUnit
+from numpy import maximum as npmax, minimum as npmin, exp as npexp
+from .. import SanUnit, WasteStream
 import numpy as np
-# import pandas as pd
 
-__all__ = ('FlatBottomCircularClarifier', 
+__all__ = ('FlatBottomCircularClarifier',
            'IdealClarifier',)
 
-def _settling_flux(X, v_max, v_max_practical, X_min, rh, rp):
-    X_star = max(X-X_min, 0)
-    v = min(v_max_practical, v_max*(np.exp(-rh*X_star) - np.exp(-rp*X_star)))
-    return X*max(v, 0)
+
+def _settling_flux(X, v_max, v_max_practical, X_min, rh, rp, n0):
+    X_star = npmax(X-X_min, n0)
+    v = npmin(v_max_practical, v_max*(npexp(-rh*X_star) - npexp(-rp*X_star)))
+    return X*npmax(v, n0)
+
+# from math import exp
+# def _settling_flux(X, v_max, v_max_practical, X_min, rh, rp, n0):
+#     X_star = max(X-X_min, 0)
+#     v = min(v_max_practical, v_max*(exp(-rh*X_star) - exp(-rp*X_star)))
+#     return X*max(v, 0)
+
 
 class FlatBottomCircularClarifier(SanUnit):
     """
@@ -35,8 +44,10 @@ class FlatBottomCircularClarifier(SanUnit):
         Influent to the clarifier. Expected number of influent is 1.
     outs : :class:`WasteStream`
         Treated effluent and sludge.
-    sludge_flow_rate : float, optional
-        Designed sludge flowrate (WAS + RAS), in [m^3/d]. The default is 2000.
+    underflow : float, optional
+        Designed recycling sludge flowrate (RAS), in [m^3/d]. The default is 2000.
+    wastage : float, optional
+        Designed wasted sludge flowrate (WAS), in [m^3/d]. The default is 385.
     surface_area : float, optional
         Surface area of the clarifier, in [m^2]. The default is 1500.
     height : float, optional
@@ -46,7 +57,7 @@ class FlatBottomCircularClarifier(SanUnit):
     feed_layer : int, optional
         The feed layer counting from top to bottom. The default is 4.
     X_threshold : float, optional
-        Threshold suspended solid cocentration, in [g/m^3]. The default is 3000.
+        Threshold suspended solid concentration, in [g/m^3]. The default is 3000.
     v_max : float, optional
         Maximum theoretical (i.e. Vesilind) settling velocity, in [m/d]. The
         default is 474.
@@ -61,11 +72,6 @@ class FlatBottomCircularClarifier(SanUnit):
     fns : float, optional
         Non-settleable fraction of the suspended solids, dimensionless. Must be within
         [0, 1]. The default is 2.28e-3.
-    t_delay : float, optional
-        Time of delay [d] for the start of change in the underflow sludge composition during 
-        dynamic simulation. When t < t_delay, the relative concentrations of particulates
-        in the underflow sludge stays the same as the set initial sludge composition. The 
-        default is 1.
 
     References
     ----------
@@ -76,17 +82,18 @@ class FlatBottomCircularClarifier(SanUnit):
     """
 
     _N_ins = 1
-    _N_outs = 2
+    _N_outs = 3
 
     def __init__(self, ID='', ins=None, outs=(), thermo=None,
-                 init_with='WasteStream', sludge_flow_rate=2000,
+                 init_with='WasteStream', underflow=2000, wastage=385,
                  surface_area=1500, height=4, N_layer=10, feed_layer=4,
                  X_threshold=3000, v_max=474, v_max_practical=250,
-                 rh=5.76e-4, rp=2.86e-3, fns=2.28e-3, t_delay=1,
-                 isdynamic=True, **kwargs):
+                 rh=5.76e-4, rp=2.86e-3, fns=2.28e-3, isdynamic=True, **kwargs):
 
         SanUnit.__init__(self, ID, ins, outs, thermo, init_with, isdynamic=isdynamic)
-        self._Qs = sludge_flow_rate
+        self._Qras = underflow
+        self._Qwas = wastage
+        self._sludge = WasteStream()
         self._V = surface_area * height
         self._A = surface_area
         self._hj = height/N_layer
@@ -98,31 +105,33 @@ class FlatBottomCircularClarifier(SanUnit):
         self._rh = rh
         self._rp = rp
         self._fns = fns
-        self._t_delay = t_delay
-        self._ODE = None
         self._solids = None
         self._solubles = None
         self._X_comp = np.zeros(len(self.components))
+        self._dX_comp = self._X_comp.copy()
         header = self._state_header
-        self._state_header = list(header[:-1]) + \
-            [f'TSS{i+1} [mg/L]' for i in range(N_layer)] + [header[-1]]
+        self._state_header = list(header) + [f'TSS{i+1} [mg/L]' for i in range(N_layer)]
         for attr, value in kwargs.items():
             setattr(self, attr, value)
 
-    def reset_cache(self):
-        '''Reset cached states.'''
-        self._state = None
-        for s in self.outs:
-            s.empty()
 
     @property
-    def sludge_flow_rate(self):
-        '''[float] The designed sludge flow rate (wasted + recycled) in m3/d.'''
-        return self._Qs
+    def underflow(self):
+        '''[float] The designed recycling sludge flow rate in m3/d.'''
+        return self._Qras
 
-    @sludge_flow_rate.setter
-    def sludge_flow_rate(self, Qs):
-        self._Qs = Qs
+    @underflow.setter
+    def underflow(self, ras):
+        self._Qras = ras
+
+    @property
+    def wastage(self):
+        '''[float] The designed wasted sludge flow rate in m3/d.'''
+        return self._Qwas
+
+    @wastage.setter
+    def wastage(self, was):
+        self._Qwas = was
 
     @property
     def V_settle(self):
@@ -131,7 +140,7 @@ class FlatBottomCircularClarifier(SanUnit):
 
     @property
     def A_settle(self):
-        '''[float] The surface area for settling in m^2, i.e., the area of the clarifer's flat bottom.'''
+        '''[float] The surface area for settling in m^2, i.e., the area of the clarifier's flat bottom.'''
         return self._A
 
     @A_settle.setter
@@ -193,7 +202,7 @@ class FlatBottomCircularClarifier(SanUnit):
 
     @property
     def X_t(self):
-        '''[float] Threshold suspended solid cocentration, in g/m^3.'''
+        '''[float] Threshold suspended solid concentration, in g/m^3.'''
         return self._X_t
 
     @X_t.setter
@@ -231,21 +240,12 @@ class FlatBottomCircularClarifier(SanUnit):
         if fns < 0 or fns > 1: raise ValueError('fns must be within [0,1].')
         self._fns = fns
 
-    @property
-    def t_delay(self):
-        return self._t_delay
-    
-    @t_delay.setter
-    def t_delay(self, T):
-        self._t_delay = max(0, T)
-
     def set_init_solubles(self, **kwargs):
         '''set the initial concentrations [mg/L] of solubles in the clarifier.'''
         Cs = np.zeros(len(self.components))
         cmpx = self.components.index
         x = self.components.x
         for k, v in kwargs.items(): Cs[cmpx(k)] = v
-        # self._solubles = np.tile(Cs*(1-x), self._N_layer)
         self._solubles = Cs*(1-x)
 
     def set_init_sludge_solids(self, **kwargs):
@@ -268,52 +268,47 @@ class FlatBottomCircularClarifier(SanUnit):
         n = self._N_layer
         x = self.components.x
         imass = self.components.i_mass
-        QCs = self._collect_ins_state()[0]
+        QCs = self._ins_QC[0]
         Q = QCs[-1]
         Z = self._solubles if self._solubles is not None \
             else QCs[:-1]*(1-x)
-            # else np.tile(QCs[:-1]*(1-x), n)
         TSS_in = sum(QCs[:-1] * x * imass)
         TSS = self._solids if self._solids is not None \
-            else np.array([TSS_in*f for f in 20**np.linspace(-1,1,n)])
+            else TSS_in*(20**np.linspace(-1,1,n))
         ZQs = np.append(Z, Q)
         self._state = np.append(ZQs, TSS)
         self._dstate = self._state * 0.
         if TSS_in != 0: self._X_comp = QCs[:-1] * x / TSS_in
 
-    def _update_state(self, arr):
-        self._state = arr
+    def _update_state(self):
+        arr = self._state
         x = self.components.x
         n = self._N_layer
-        Q = arr[-(1+n)]
-        Q_e = Q - self._Qs
+        Q_e = arr[-(1+n)] - self._Qras - self._Qwas
         Z = arr[:len(x)]
-        X_composition = self._X_comp # (m, ), mg COD/ mg TSS
+        inf, = self.ins
+        imass = self.components.i_mass
+        C_in = inf.state[:-1]
+        X_composition = self._X_comp = C_in*x/sum(C_in*imass*x)
         X_e = arr[-n] * X_composition
-        X_s = arr[-1] * X_composition
-        eff, slg = self._outs
-        if eff._state is None: eff._state = np.append(Z+X_e, Q_e)
-        else: 
-            eff._state[:-1] = Z+X_e
-            eff._state[-1] = Q_e
-        if slg._state is None: slg._state = np.append(Z+X_s, self._Qs)
+        C_s = Z + arr[-1] * X_composition
+        eff, ras, was = self._outs
+        if eff.isproduct() and eff.state is None: 
+            eff.state = np.append(Z+X_e, Q_e)
         else:
-            slg._state[:-1] = Z+X_s
-
-    # def _state_locator(self, arr):
-    #     x = self.components.x
-    #     n = self._N_layer
-    #     dct = {}
-    #     Q = arr[-(1+n)]
-    #     Q_e = Q - self._Qs
-    #     Z = arr[:len(x)]
-    #     X_composition = self._X_comp # (m, ), mg COD/ mg TSS
-    #     X_e = arr[-n] * X_composition
-    #     X_s = arr[-1] * X_composition
-    #     dct[self.ID] = arr
-    #     dct[self.outs[0].ID] = np.append(Z+X_e, Q_e)
-    #     dct[self.outs[1].ID] = np.append(Z+X_s, self._Qs)        
-    #     return dct
+            eff.state[:-1] = Z+X_e   # not sure if this works for a setter
+            eff.state[-1] = Q_e
+        #!!! might need to enable dynamic sludge volume flows
+        if ras.isproduct() and ras.state is None: 
+            ras.state = np.append(C_s, self._Qras)
+        else: 
+            ras.state[:-1] = C_s
+            ras.state[-1] = self._Qras
+        if was.isproduct() and was.state is None: 
+            was.state = np.append(C_s, self._Qwas)
+        else: 
+            was.state[:-1] = C_s
+            was.state[-1] = self._Qwas
 
     def _update_dstate(self):
         arr = self._dstate
@@ -321,54 +316,51 @@ class FlatBottomCircularClarifier(SanUnit):
         n = self._N_layer
         dQ = arr[-(1+n)]
         dZ = arr[:len(x)]
+        TSS_e, TSS_s = self._state[-n], self._state[-1]
         X_composition = self._X_comp # (m, ), mg COD/ mg TSS
-        try: dX_e = arr[-n] * X_composition
-        except: breakpoint()
-        dX_s = arr[-1] * X_composition
-        eff, slg = self._outs
-        if eff._dstate is None: eff._dstate = np.append(dZ+dX_e, dQ)
-        else: 
-            eff._dstate[:-1] = dZ+dX_e
-            eff._dstate[-1] = dQ
-        if slg._dstate is None: slg._dstate = np.append(dZ+dX_s, 0.)
+        dX_composition = self._dX_comp
+        dC_e = dZ + arr[-n] * X_composition + dX_composition * TSS_e
+        dC_s = dZ + arr[-1] * X_composition + dX_composition * TSS_s
+        eff, ras, was = self._outs
+        if eff.isproduct() and eff.dstate is None: 
+            eff.dstate = np.append(dC_e, dQ)
         else:
-            slg._dstate[:-1] = dZ+dX_s
-    
-    # def _dstate_locator(self, arr):
-    #     x = self.components.x
-    #     n = self._N_layer
-    #     dct = {}
-    #     dQ = arr[-(1+n)]
-    #     dZ = arr[:len(x)]
-    #     X_composition = self._X_comp # (m, ), mg COD/ mg TSS
-    #     dX_e = arr[-n] * X_composition
-    #     dX_s = arr[-1] * X_composition
-    #     dct[self.ID] = arr
-    #     dct[self.outs[0].ID] = np.append(dZ+dX_e, dQ)
-    #     dct[self.outs[1].ID] = np.append(dZ+dX_s, 0)
-    #     return dct
-
-    # def _load_state(self):
-    #     '''returns a dictionary of values of state variables within the clarifer and in the output streams.'''
-    #     if self._state is None: self._init_state()
-    #     return {self.ID: self._state}
+            eff.dstate[:-1] = dC_e # not sure if this works for a setter
+            eff.dstate[-1] = dQ
+        #!!! might need to enable dynamic sludge volume flows
+        if ras.isproduct() and ras.dstate is None: 
+            ras.dstate = np.append(dC_s, 0.)
+        else: 
+            ras.dstate[:-1] = dC_s
+        if was.isproduct() and was.dstate is None: 
+            was.dstate = np.append(dC_s, 0.)
+        else: 
+            was.dstate[:-1] = dC_s
 
     def _run(self):
         '''only to converge volumetric flows.'''
         inf, = self.ins
+        sludge = self._sludge
         Q_in = inf.get_total_flow('m3/d')
-        eff, sludge = self.outs
-        Q_s = self._Qs
-        Q_e = max(Q_in - Q_s, 0)
-        inf.split_to(eff, sludge, split=Q_e/Q_in)
+        eff, ras, was = self.outs
+        Q_ras = self._Qras
+        Q_was = self._Qwas
+        s_e = 1 - (Q_ras+Q_was)/Q_in
+        inf.split_to(eff, sludge, s_e)
+        sludge.split_to(ras, was, Q_ras/(Q_ras+Q_was))
 
+    def get_retained_mass(self, biomass_IDs):
+        cmps = self.components
+        tss = self._state[-self._N_layer:].mean()
+        mass = cmps.i_mass * self._X_comp * tss
+        return self._V * mass[cmps.indices(biomass_IDs)].sum()
 
     @property
     def ODE(self):
         if self._ODE is None:
             self._compile_ODE()
         return self._ODE
-    
+
     def _compile_ODE(self):
         n = self._N_layer
         jf = self._feed_layer - 1
@@ -376,73 +368,91 @@ class FlatBottomCircularClarifier(SanUnit):
         m = len(x)
         imass = self.components.i_mass
         fns = self._fns
-        vmax = self._v_max
-        vmaxp = self._v_max_p
-        rh = self._rh
-        rp = self._rp
-        X_t = self._X_t
-        A = self._A
-        hj = self._hj
-        Q_s = self._Qs
-        T = self._t_delay
+        Q_s = self._Qras + self._Qwas
+
+        dQC = self._dstate
+        dX_comp = self._dX_comp
+        _update_dstate = self._update_dstate
+
+        nzeros = np.zeros(n)
+        Q_jout = nzeros.copy()
+        X_rolled = nzeros.copy()
+        X_min_arr = nzeros.copy()
+        settle_out = nzeros.copy()
+        settle_in = nzeros.copy()
+
+        # Make these constants into arrays so it'll be faster in `dy_dt`
+        vmax_arr = np.full_like(nzeros, self._v_max)
+        vmaxp_arr = np.full_like(nzeros, self._v_max_p)
+        rh_arr = np.full_like(nzeros, self._rh)
+        rp_arr = np.full_like(nzeros, self._rp)
+        func_vx = lambda x_arr, xmin_arr : _settling_flux(x_arr, vmax_arr, vmaxp_arr, xmin_arr, rh_arr, rp_arr, nzeros)
         
+        A, hj, V = self._A, self._hj, self._V
+        A_arr = np.full_like(nzeros, A)
+        hj_arr = np.full_like(nzeros, hj)
+        J = np.zeros(n-1)
+        X_t_arr = np.full(jf, self._X_t)
+        Q_in_arr = np.zeros(m)
+        V_arr = np.full(m, V)
+
         def dy_dt(t, QC_ins, QC, dQC_ins):
-            dQC = self._dstate
             dQC[-(n+1)] = dQC_ins[0,-1]
             Q_in = QC_ins[0,-1]
             Q_e = Q_in - Q_s
             C_in = QC_ins[0,:-1]
+            dC_in = dQC_ins[0,:-1]
             Z_in = C_in*(1-x)
             X_in = sum(C_in*imass*x)           # influent TSS
-            if X_in != 0 and t >= T: self._X_comp = C_in * x / X_in     # g COD/g TSS for solids in influent
-            X_min = X_in * fns
+            dX_in = sum(dC_in*imass*x)
+            X_min_arr[:] = X_in * fns
             X = QC[-n:]                        # (n, ), TSS for each layer
             Z = QC[:m] * (1-x)
             #***********TSS*************
-            Q_jout = np.array([Q_e if j < jf else Q_in if j == jf else Q_s for j in range(n)])
-            flow_out = X*Q_jout
-            flow_in = np.array([Q_e*X[j+1] if j < jf else Q_in*X_in if j == jf else Q_s*X[j-1] for j in range(n)])
-            VX = [_settling_flux(xj, vmax, vmaxp, X_min, rh, rp) for xj in X]
-            J = [VX[j] if X[j+1] <= X_t and j < jf else min(VX[j], VX[j+1]) for j in range(n-1)]
-            settle_out = np.array(J + [0])
-            settle_in = np.array([0] + J)
-            dQC[-n:] = ((flow_in - flow_out)/A + settle_in - settle_out)/hj        # (n,)
+            Q_jout[:jf] = Q_e
+            Q_jout[jf] = Q_in
+            Q_jout[jf+1:] = Q_s
+            flow_out = X * Q_jout
+            X_rolled[:jf] = X[1: jf+1]
+            X_rolled[jf] = X_in
+            X_rolled[jf+1:] = X[jf: -1]
+            flow_in = X_rolled * Q_jout
+            VX = func_vx(X, X_min_arr)
+            J[:] = npmin(VX[:-1], VX[1:])
+            condition = (X_rolled[:jf]<X_t_arr)
+            J[:jf][condition] = VX[:jf][condition]
+            settle_out[:-1] = J
+            settle_in[1:] = J
+            dQC[-n:] = ((flow_in - flow_out)/A_arr + settle_in - settle_out)/hj_arr       # (n,)
             #*********solubles**********
-            dQC[:m] = Q_in*(Z_in - Z)/A/(hj*n)
-            self._update_dstate()
-            # return dQC
-        
+            Q_in_arr[:] = Q_in
+            dQC[:m] = Q_in_arr*(Z_in - Z)/V_arr
+            # instrumental variables
+            dX_comp[:] = (dC_in * X_in - dX_in * C_in) * x / X_in**2
+            _update_dstate()
+
         self._ODE = dy_dt
-                
-    
-    def _define_outs(self):
-        dct_y = self._state_locator(self._state)
-        for out in self.outs:
-            Q = dct_y[out.ID][-1]
-            Cs = dict(zip(self.components.IDs, dct_y[out.ID][:-1]))
-            Cs.pop('H2O', None)
-            out.set_flow_by_concentration(Q, Cs, units=('m3/d', 'mg/L'))
 
     def _design(self):
         pass
 
 
 class IdealClarifier(SanUnit):
-    
+
     _N_ins = 1
     _N_outs = 2
 
     def __init__(self, ID='', ins=None, outs=(), thermo=None,
                  sludge_flow_rate=2000, solids_removal_efficiency=.995,
-                 sludge_MLSS=None, isdynamic=False, init_with='WasteStream', 
+                 sludge_MLSS=None, isdynamic=False, init_with='WasteStream',
                  F_BM_default=None, **kwargs):
-        
+
         SanUnit.__init__(self, ID, ins, outs, thermo, isdynamic=isdynamic,
                          init_with=init_with, F_BM_default=F_BM_default)
         self.sludge_flow_rate = sludge_flow_rate
         self.solids_removal_efficiency = solids_removal_efficiency
         self.sludge_MLSS = sludge_MLSS
-        
+
     @property
     def sludge_flow_rate(self):
         '''[float] The designed sludge flow rate (wasted + recycled) in m3/d.'''
@@ -457,7 +467,7 @@ class IdealClarifier(SanUnit):
     @property
     def solids_removal_efficiency(self):
         return self._e_rmv
-    
+
     @solids_removal_efficiency.setter
     def solids_removal_efficiency(self, f):
         if f is not None:
@@ -466,27 +476,27 @@ class IdealClarifier(SanUnit):
             self._e_rmv = f
         elif self.ins[0].isempty(): self._e_rmv = None
         else: self._e_rmv = self._calc_ermv()
-    
+
     @property
     def sludge_MLSS(self):
         return self._MLSS
-    
+
     @sludge_MLSS.setter
     def sludge_MLSS(self, MLSS):
         if MLSS is not None: self._MLSS = MLSS
         elif self.ins[0].isempty(): self._MLSS = None
         else: self._MLSS = self._calc_SS()[1]
-    
+
     def _calc_Qs(self, TSS_in=None, Q_in=None):
         if Q_in is None: Q_in = self.ins[0].get_total_flow('m3/d')
         if TSS_in is None: TSS_in = self.ins[0].get_TSS()
         return Q_in*TSS_in*self._e_rmv/(self._MLSS-TSS_in)
-    
+
     def _calc_ermv(self, TSS_in=None, Q_in=None):
         if Q_in is None: Q_in = self.ins[0].get_total_flow('m3/d')
         if TSS_in is None: TSS_in = self.ins[0].get_TSS()
-        return self._Qs*(self._MLSS-TSS_in)/TSS_in/(Q_in-self._Qs)        
-    
+        return self._Qs*(self._MLSS-TSS_in)/TSS_in/(Q_in-self._Qs)
+
     def _calc_SS(self, SS_in=None, Q_in=None):
         if Q_in is None: Q_in = self.ins[0].get_total_flow('m3/d')
         if SS_in is None: SS_in = self.ins[0].get_TSS()
@@ -494,7 +504,7 @@ class IdealClarifier(SanUnit):
         Qs = self._Qs
         Qe = Q_in - Qs
         return SS_e, (Q_in*SS_in - Qe*SS_e)/Qs
-    
+
     def _run(self):
         inf, = self.ins
         eff, sludge = self.outs
@@ -502,7 +512,7 @@ class IdealClarifier(SanUnit):
         Q_in = inf.get_total_flow('m3/d')
         TSS_in = (inf.conc*cmps.x*cmps.i_mass).sum()
         params = (Qs, e_rmv, MLSS) = self._Qs, self._e_rmv, self._MLSS
-        if sum([i is None for i in params]) > 1: 
+        if sum([i is None for i in params]) > 1:
             raise RuntimeError('must specify two of the following parameters: '
                                'sludge_flow_rate, solids_removal_efficiency, sludge_MLSS')
         if Qs is None:
@@ -522,7 +532,6 @@ class IdealClarifier(SanUnit):
         Cs.pop('H2O', None)
         eff.set_flow_by_concentration(Q_in-Qs, Ce, units=('m3/d', 'mg/L'))
         sludge.set_flow_by_concentration(Qs, Cs, units=('m3/d', 'mg/L'))
-    
+
     def _design(self):
-        pass
-        
\ No newline at end of file
+        pass
\ No newline at end of file
