@@ -5,7 +5,7 @@
 QSDsan: Quantitative Sustainable Design for sanitation and resource recovery systems
 
 This module is developed by:
-    Yalin Li <zoe.yalin.li@gmail.com>
+    Yalin Li <mailto.yalin.li@gmail.com>
 
 This module is under the University of Illinois/NCSA Open Source License.
 Please refer to https://github.com/QSD-Group/QSDsan/blob/main/LICENSE.txt
@@ -26,6 +26,9 @@ class Trucking(SanUnit):
     For transportation of materials with considerations on material loss
     based on `Trimmer et al. <https://doi.org/10.1021/acs.est.0c03296>`_
 
+    To enable life cycle assessment, the following impact items should be pre-constructed:
+    `Trucking`.
+
     Parameters
     ----------
     load_type : str
@@ -48,6 +51,9 @@ class Trucking(SanUnit):
         If material loss occurs during transportation.
     loss_ratio : float or dict
         Fractions of material losses due to transportation.
+    price_ratio : float.
+        Calculated transportation cost will be multiplied by this number
+        to consider the effect in cost difference from different locations.
 
     Examples
     --------
@@ -66,18 +72,19 @@ class Trucking(SanUnit):
                  distance=1., distance_unit='km',
                  interval=1., interval_unit='hr',
                  fee=0., fee_unit=currency,
-                 if_material_loss=True, loss_ratio=0.02):
+                 if_material_loss=True, loss_ratio=0.02, price_ratio=1.):
         SanUnit.__init__(self, ID, ins, outs, thermo, init_with)
         self.single_truck = single = \
-            Transportation('single_truck', item='Trucking',
+            Transportation('single_truck', linked_unit=self, item='Trucking',
                            load_type=load_type, load=load, load_unit=load_unit,
                            distance=distance, distance_unit=distance_unit,
                            interval=interval, interval_unit=interval_unit)
-        total = single.copy('total_truck')
+        total = single.copy('total_truck', linked_unit=self)
         self.transportation = (total,)
         self._update_fee(fee, fee_unit)
         self.if_material_loss = if_material_loss
         self.loss_ratio = loss_ratio
+        self.price_ratio = price_ratio
 
     _N_ins = 1
     _N_outs = 2
@@ -117,7 +124,7 @@ class Trucking(SanUnit):
         total, = self.transportation
         copy_attr(total, single, skip=('_ID', '_item')) # in case attributes have been updated
         total.load = single.load * N
-        self._add_OPEX = {'Total fee': self.fee/total.interval*N}
+        self._add_OPEX = {'Total fee': self.fee/total.interval*N*self.price_ratio}
 
     @property
     def fee(self):
