@@ -5,7 +5,7 @@
 QSDsan: Quantitative Sustainable Design for sanitation and resource recovery systems
 
 This module is developed by:
-    Yalin Li <zoe.yalin.li@gmail.com>
+    Yalin Li <mailto.yalin.li@gmail.com>
 
 This module is under the University of Illinois/NCSA Open Source License.
 Please refer to https://github.com/QSD-Group/QSDsan/blob/main/LICENSE.txt
@@ -15,10 +15,11 @@ for license details.
 
 # %%
 
-__all__ = ('format_number', )
+__all__ = ('format_number', 'format_str',)
 
 
 def format_number(number):
+    '''Pretty print number based on its magnitude.'''
     if not number:
         return 'None'
 
@@ -36,4 +37,11 @@ def format_number(number):
         return str(int(number))
 
     else:
-        return str(round(number, 2))
\ No newline at end of file
+        return str(round(number, 2))
+
+
+def format_str(string):
+    '''Replace space and "-" with "_".'''
+    string = string.replace(' ', '_')
+    string = string.replace('-', '_')
+    return string
\ No newline at end of file
